@@ -23,8 +23,9 @@
 #include <config.h>
 #include <string.h>
 #include <unistd.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glade/glade.h>
@@ -61,120 +62,105 @@ destroy_cb (GtkWidget *w,
 	SaveImageData *data;
 
 	pixbuf = g_object_get_data (G_OBJECT (file_sel), "pixbuf");
-	g_object_unref (pixbuf);
+	if (pixbuf != NULL)
+		g_object_unref (pixbuf);
 
 	data = g_object_get_data (G_OBJECT (file_sel), "data");
 	g_free (data);
 }
 
 
-static void 
-file_save_cancel_cb (GtkWidget *w,
-		     GtkWidget *file_sel)
+static void
+file_save_cancel_cb (GtkDialog *file_sel,
+		     int        button_number,
+		     gpointer  *userdata)
+
 {
-	SaveImageData *data;	
+	SaveImageData *data;
 
 	data = g_object_get_data (G_OBJECT (file_sel), "data");
-	if (data->done_func != NULL) 
+	if (data->done_func != NULL)
 		(*data->done_func) (NULL, data->done_data);
-	gtk_widget_destroy (file_sel);
+	gtk_widget_destroy (GTK_WIDGET (file_sel));
 }
 
 
-static void 
-file_save_ok_cb (GtkWidget *w,
-		 GtkWidget *file_sel)
+static gboolean
+save_image (GtkWindow       *parent,
+     	    const char      *filename,
+	    const char      *mime_type,
+	    GdkPixbuf       *pixbuf,
+	    SaveImageData   *data,
+	    GtkDialog       *file_sel)
 {
-	static char   *mime_types[4] = {"image/jpeg", "image/png", "image/tga", "image/tiff"};
-	GtkWindow     *parent;
-	GtkWidget     *opt_menu;
-	GdkPixbuf     *pixbuf;
-	char          *filename = NULL;
-	char          *dir;
-	gboolean       file_exists;
-	const char    *mime_type;
-	int            idx;
-	gboolean       image_saved = FALSE;
-	SaveImageData *data;	
-
-	parent = g_object_get_data (G_OBJECT (file_sel), "parent_window");
-	pixbuf = g_object_get_data (G_OBJECT (file_sel), "pixbuf");
-	filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel)));
+	char     *dir;
+	gboolean  file_exists;
+	gboolean  image_saved = FALSE;
 
 	if (filename == NULL)
-		return;
+		return FALSE;
 
 	/* Check permissions */
 
 	dir = remove_level_from_path (filename);
-	if (access (dir, R_OK | W_OK | X_OK) != 0) {
+	if (! check_permissions (dir, R_OK | W_OK | X_OK)) {
 		char *utf8_path;
-		utf8_path = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_display_name (dir);
 		_gtk_error_dialog_run (parent,
 				       _("You don't have the right permissions to create images in the folder \"%s\""),
 				       utf8_path);
 		g_free (utf8_path);
 		g_free (dir);
-		g_free (filename);
-		return;
+		return FALSE;
 	}
 	g_free (dir);
 
 	file_exists = path_is_file (filename);
 
-	if (file_exists) { 
+	if ((file_sel != NULL) && file_exists) {
 		char      *message;
 		GtkWidget *d;
 		int        r;
 
 		message = g_strdup_printf (_("An image named \"%s\" is already present. Do you want to overwrite it?"), file_name_from_path (filename));
-		d = _gtk_yesno_dialog_new (parent, 
+		d = _gtk_yesno_dialog_new (parent,
 					   GTK_DIALOG_MODAL,
 					   message,
 					   GTK_STOCK_NO,
 					   GTK_STOCK_YES);
 		g_free (message);
-		
+
 		r = gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
 
-		if (r != GTK_RESPONSE_YES) {
-			g_free (filename);
-			return;
-		}
+		if (r != GTK_RESPONSE_YES)
+			return FALSE;
 	}
 
-	gtk_widget_hide (file_sel);
-
-	opt_menu = g_object_get_data (G_OBJECT (file_sel), "opt_menu");
-	idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_menu));
-
-	if (idx == IMAGE_TYPE_AUTOMATIC)
-		mime_type = gnome_vfs_mime_type_from_name (filename);
-	else
-		mime_type = mime_types [idx - 2];
+	if (file_sel != NULL)
+		gtk_widget_hide (GTK_WIDGET (file_sel));
 
 	if ((mime_type != NULL)
 	    && is_mime_type_writable (mime_type)) {
 		GError      *error = NULL;
-		const char  *image_type = mime_type + 6;
+		const char  *image_type = mime_type + strlen("image/");
 		char       **keys = NULL;
 		char       **values = NULL;
-		
+
 		if (dlg_save_options (parent,
 				      image_type,
 				      &keys,
 				      &values)) {
-			if (! _gdk_pixbuf_savev (pixbuf, 
-						 filename, 
-						 image_type, 
+			if (! _gdk_pixbuf_savev (pixbuf,
+						 filename,
+						 image_type,
 						 keys, values,
-						 &error)) 
+						 &error))
 				_gtk_error_dialog_from_gerror_run (parent, &error);
-			else 
+			else
 				image_saved = TRUE;
 		}
-		
+
 		g_strfreev (keys);
 		g_strfreev (values);
 	} else
@@ -182,17 +168,61 @@ file_save_ok_cb (GtkWidget *w,
 				       _("Image type not supported: %s"),
 				       mime_type);
 
-	if (! image_saved) {
-		g_free (filename);
-		filename = NULL;
+	if (data->done_func != NULL) {
+		if (! image_saved)
+			filename = NULL;
+		(*data->done_func) (filename, data->done_data);
 	}
 
+	if (file_sel != NULL)
+		gtk_widget_destroy (GTK_WIDGET (file_sel));
+
+	return TRUE;
+}
+
+
+static void
+file_save_ok_cb (GtkDialog *file_sel,
+		 int        button_number,
+		 gpointer  *userdata)
+
+{
+	static char   *mime_types[4] = {"image/jpeg", "image/png", "image/tga", "image/tiff"};
+	GtkWindow     *parent;
+	GtkWidget     *opt_menu;
+	GdkPixbuf     *pixbuf;
+	char          *filename = NULL;
+	const char    *mime_type = NULL;
+	int            idx;
+	SaveImageData *data;
+
+	parent = g_object_get_data (G_OBJECT (file_sel), "parent_window");
+	pixbuf = g_object_get_data (G_OBJECT (file_sel), "pixbuf");
 	data = g_object_get_data (G_OBJECT (file_sel), "data");
-	if (data->done_func != NULL) 
-		(*data->done_func) (filename, data->done_data);
-	
+	filename = g_strdup (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel)));
+
+	opt_menu = g_object_get_data (G_OBJECT (file_sel), "opt_menu");
+	idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_menu));
+	if (idx == IMAGE_TYPE_AUTOMATIC)
+		mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
+	else
+		mime_type = mime_types [idx - 2];
+
+	save_image (parent, filename, mime_type, pixbuf, data, file_sel);
+
 	g_free (filename);
-	gtk_widget_destroy (file_sel);
+}
+
+
+static void
+file_save_response_cb (GtkDialog *file_sel,
+		       int        button_number,
+		       gpointer  *userdata)
+{
+	if (button_number == GTK_RESPONSE_ACCEPT)
+		file_save_ok_cb (file_sel, button_number, userdata);
+	else
+		file_save_cancel_cb (file_sel, button_number, userdata);
 }
 
 
@@ -225,37 +255,41 @@ build_file_type_menu (void)
 
 
 void
-dlg_save_image (GtkWindow       *parent,
-		const char      *current_folder,
-		GdkPixbuf       *pixbuf,
-		ImageSavedFunc   done_func,
-		gpointer         done_data)
+dlg_save_image_as (GtkWindow       *parent,
+		   const char      *uri,
+		   GdkPixbuf       *pixbuf,
+		   ImageSavedFunc   done_func,
+		   gpointer         done_data)
 {
 	SaveImageData *data;
-	GtkWidget *file_sel;
-	GtkWidget *vbox;
-	GtkWidget *hbox;
-	GtkWidget *opt_menu;
-	GtkWidget *menu;
-	char      *path;
+	GtkWidget     *file_sel;
+	GtkWidget     *vbox;
+	GtkWidget     *hbox;
+	GtkWidget     *opt_menu;
+	GtkWidget     *menu;
 
 	g_return_if_fail (pixbuf != NULL);
 
-	file_sel = gtk_file_selection_new (_("Save Image"));
-	
+	file_sel = gtk_file_chooser_dialog_new(_("Save Image"), parent,
+					       GTK_FILE_CHOOSER_ACTION_SAVE,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					       NULL);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (file_sel), GTK_RESPONSE_ACCEPT);
+
 	/**/
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
-	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (file_sel)->action_area), 
-			    vbox, 
-			    TRUE, TRUE, 0);
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel),
+					   vbox);
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 6);
 
-	gtk_box_pack_start (GTK_BOX (hbox), 
-			    gtk_label_new (_("Image type:")), 
+	gtk_box_pack_start (GTK_BOX (hbox),
+			    gtk_label_new (_("Image type:")),
 			    FALSE, FALSE, 0);
 
 	opt_menu = gtk_option_menu_new ();
@@ -267,15 +301,10 @@ dlg_save_image (GtkWindow       *parent,
 
 	/**/
 
-	if (current_folder != NULL)
-		path = g_strdup (current_folder);
+	if (uri != NULL)
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_sel), uri);
 	else
-		path = g_strconcat (g_get_home_dir (),
-				    "/",
-				    NULL);
-
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), path);
-	g_free (path);
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_sel), get_home_uri ());
 
 	g_object_ref (pixbuf);
 
@@ -288,21 +317,16 @@ dlg_save_image (GtkWindow       *parent,
 	g_object_set_data (G_OBJECT (file_sel), "data", data);
 	g_object_set_data (G_OBJECT (file_sel), "opt_menu", opt_menu);
 
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked", 
-			  G_CALLBACK (file_save_ok_cb), 
-			  file_sel);
-
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-			  "clicked", 
-			  G_CALLBACK (file_save_cancel_cb),
-			  file_sel);
+	g_signal_connect (GTK_DIALOG (file_sel),
+			  "response",
+			  G_CALLBACK (file_save_response_cb),
+			  NULL);
 
 	g_signal_connect (G_OBJECT (file_sel),
-			  "destroy", 
+			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  file_sel);
-    
+
 	if (parent != NULL) {
 		gtk_window_set_transient_for (GTK_WINDOW (file_sel), parent);
 		gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
@@ -312,9 +336,33 @@ dlg_save_image (GtkWindow       *parent,
 }
 
 
+
+void
+dlg_save_image (GtkWindow       *parent,
+		const char      *filename,
+		GdkPixbuf       *pixbuf,
+		ImageSavedFunc   done_func,
+		gpointer         done_data)
+{
+	SaveImageData *data;
+	const char    *mime_type = NULL;
+
+	if (filename == NULL)
+		return;
+
+	data = g_new0 (SaveImageData, 1);
+	data->done_func = done_func;
+	data->done_data = done_data;
+
+	mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
+
+	save_image (parent, filename, mime_type, pixbuf, data, NULL);
+}
+
+
 
 
-#define GLADE_FILE     "gthumb_convert.glade"
+#define GLADE_FILE "gthumb_convert.glade"
 
 
 typedef struct {
@@ -339,9 +387,9 @@ get_type_from_name (const char *name)
 	int i;
 
 	for (i = 0; types_table[i].name != NULL; i++)
-		if (strcmp (types_table[i].name, name) == 0) 
+		if (strcasecmp (types_table[i].name, name) == 0)
 			return types_table + i;
-	
+
 	return NULL;
 }
 
@@ -359,17 +407,17 @@ dlg_save_options (GtkWindow    *parent,
 	TypeEntry  *type;
 
 	*keys   = NULL;
-	*values = NULL;	
+	*values = NULL;
 
 	if (image_type == NULL) {
 		g_warning ("Invalid image type\n");
 		return FALSE;
 	}
 
-	if (strncmp (image_type, "x-", 2) == 0) 
+	if (strncmp (image_type, "x-", 2) == 0)
 		image_type += 2;
 
-	type = get_type_from_name (image_type); 
+	type = get_type_from_name (image_type);
 	if (type == NULL) {
 		g_warning ("Invalid image type\n");
 		return FALSE;
@@ -400,48 +448,48 @@ dlg_save_options (GtkWindow    *parent,
 		ivalue = eel_gconf_get_integer (PREF_JPEG_QUALITY, 85);
 		widget = glade_xml_get_widget (gui, "jpeg_quality_hscale");
 		gtk_range_set_value (GTK_RANGE (widget), (double) ivalue);
-		
+
 		/**/
 
 		ivalue = eel_gconf_get_integer (PREF_JPEG_SMOOTHING, 0);
 		widget = glade_xml_get_widget (gui, "jpeg_smooth_hscale");
 		gtk_range_set_value (GTK_RANGE (widget), (double) ivalue);
-		
+
 		/**/
 
 		ivalue = eel_gconf_get_boolean (PREF_JPEG_OPTIMIZE, TRUE);
 		widget = glade_xml_get_widget (gui, "jpeg_optimize_checkbutton");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), ivalue);
-		
+
 		/**/
 
 		ivalue = eel_gconf_get_boolean (PREF_JPEG_PROGRESSIVE, FALSE);
 		widget = glade_xml_get_widget (gui, "jpeg_progressive_checkbutton");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), ivalue);
 		break;
-		
+
 	case IMAGE_TYPE_PNG:
 		break;
-		
+
 	case IMAGE_TYPE_TIFF:
-		svalue = eel_gconf_get_string (PREF_TIFF_COMPRESSION, "deflate"); 
+		svalue = eel_gconf_get_string (PREF_TIFF_COMPRESSION, "deflate");
 
 		if (svalue == NULL)
 			widget = NULL;
-		else if (strcmp (svalue, "none") == 0) 
+		else if (strcmp (svalue, "none") == 0)
 			widget = glade_xml_get_widget (gui, "tiff_comp_none_radiobutton");
-		else if (strcmp (svalue, "deflate") == 0) 
+		else if (strcmp (svalue, "deflate") == 0)
 			widget = glade_xml_get_widget (gui, "tiff_comp_deflate_radiobutton");
-		else if (strcmp (svalue, "jpeg") == 0) 
+		else if (strcmp (svalue, "jpeg") == 0)
 			widget = glade_xml_get_widget (gui, "tiff_comp_jpeg_radiobutton");
 		else
 			widget = NULL;
-		
+
 		if (widget != NULL)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
 		g_free (svalue);
-		
+
 		/**/
 
 		ivalue = eel_gconf_get_integer (PREF_TIFF_HORIZONTAL_RES, 72);
@@ -453,9 +501,9 @@ dlg_save_options (GtkWindow    *parent,
 		ivalue = eel_gconf_get_integer (PREF_TIFF_VERTICAL_RES, 72);
 		widget = glade_xml_get_widget (gui, "tiff_vdpi_spinbutton");
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), (double) ivalue);
-		
+
 		break;
-		
+
 	case IMAGE_TYPE_TGA:
 		ivalue = eel_gconf_get_boolean (PREF_TGA_RLE_COMPRESSION, TRUE);
 		widget = glade_xml_get_widget (gui, "tga_rle_compression_checkbutton");
@@ -477,7 +525,7 @@ dlg_save_options (GtkWindow    *parent,
 
 		*keys   = g_malloc (sizeof (char*) * (type->n_params + 1));
 		*values = g_malloc (sizeof (char*) * (type->n_params + 1));
-		
+
 		switch (type->type) {
 		case IMAGE_TYPE_JPEG:
 			(*keys)[i] = g_strdup ("quality");
@@ -522,10 +570,10 @@ dlg_save_options (GtkWindow    *parent,
 			eel_gconf_set_boolean (PREF_JPEG_PROGRESSIVE, ivalue);
 
 			break;
-			
+
 		case IMAGE_TYPE_PNG:
 			break;
-			
+
 		case IMAGE_TYPE_TIFF:
 			(*keys)[i] = g_strdup ("compression");
 
@@ -544,7 +592,7 @@ dlg_save_options (GtkWindow    *parent,
 			if (ivalue)
 				(*values)[i] = g_strdup_printf ("jpeg");
 
-			eel_gconf_set_string (PREF_TIFF_COMPRESSION, (*values)[i]); 
+			eel_gconf_set_string (PREF_TIFF_COMPRESSION, (*values)[i]);
 
 			/**/
 
@@ -569,7 +617,7 @@ dlg_save_options (GtkWindow    *parent,
 			eel_gconf_set_integer (PREF_TIFF_VERTICAL_RES, ivalue);
 
 			break;
-			
+
 		case IMAGE_TYPE_TGA:
 			(*keys)[i] = g_strdup ("compression");
 
@@ -579,7 +627,7 @@ dlg_save_options (GtkWindow    *parent,
 				(*values)[i] = g_strdup_printf ("rle");
 			else
 				(*values)[i] = g_strdup_printf ("none");
-			
+
 			eel_gconf_set_boolean (PREF_TGA_RLE_COMPRESSION, strcmp ((*values)[i], "rle") == 0);
 
 			break;
@@ -596,7 +644,7 @@ dlg_save_options (GtkWindow    *parent,
 		retval = FALSE;
 
 	/**/
-	    
+
 	gtk_widget_destroy (dialog);
 	g_object_unref (gui);
 
