@@ -29,12 +29,11 @@
 #include "file-utils.h"
 #include "comments.h"
 
-
 #define MAX_COMMENT_LEN 60
 
 
 FileData *
-file_data_new (const char       *path, 
+file_data_new (const char       *path,
 	       GnomeVFSFileInfo *info)
 {
 	FileData *fd;
@@ -50,6 +49,14 @@ file_data_new (const char       *path,
 		fd->ctime = info->ctime;
 		fd->mtime = info->mtime;
 	}
+
+	/* The Exif DateTime tag is only recorded on an as-needed basis during
+	   DateTime sorts. The tag in memory is refreshed if the file mtime has
+	   changed, so it is recorded as well. */
+
+	fd->exif_data_loaded = FALSE;
+	fd->exif_time = 0;
+
 	fd->error = FALSE;
 	fd->thumb = FALSE;
 	fd->comment = g_strdup ("");
@@ -67,6 +74,9 @@ file_data_update (FileData *fd)
 
 	g_return_if_fail (fd != NULL);
 
+	fd->error = FALSE;
+	fd->thumb = FALSE;
+
 	escaped = escape_uri (fd->path);
 	info = gnome_vfs_file_info_new ();
 	result = gnome_vfs_get_file_info (escaped,
@@ -76,7 +86,11 @@ file_data_update (FileData *fd)
 	g_free (escaped);
 
 	if (result != GNOME_VFS_OK) {
-		g_warning ("Cannot get info of file : %s\n", fd->path);
+		fd->error = TRUE;
+		fd->size = 0L;
+		fd->mtime = 0;
+		fd->ctime = 0;
+		fd->exif_data_loaded = FALSE;
 		return;
 	}
 
@@ -88,6 +102,7 @@ file_data_update (FileData *fd)
 	fd->size = info->size;
 	fd->mtime = info->mtime;
 	fd->ctime = info->ctime;
+	fd->exif_data_loaded = FALSE;
 
 	gnome_vfs_file_info_unref (info);
 }
@@ -182,3 +197,10 @@ file_data_list_free (GList *list)
 		g_list_free (list);
 	}
 }
+
+
+const char *
+file_data_local_path (FileData *fd)
+{
+	return get_file_path_from_uri (fd->path);
+}
