@@ -38,6 +38,8 @@ typedef struct {
 	GnomeVFSFileSize    size;
 	time_t              ctime;
 	time_t              mtime;
+	guint               exif_data_loaded : 1;
+	time_t		    exif_time;
 	guint               error : 1;     /* Whether an error occurred loading
 					    * this file. */
 	guint               thumb : 1;     /* Whether we have a thumb of this
@@ -46,20 +48,17 @@ typedef struct {
 } FileData;
 
 
-FileData *   file_data_new               (const char       *path, 
+FileData *   file_data_new               (const char       *path,
 					  GnomeVFSFileInfo *info);
-
 void         file_data_ref               (FileData         *fd);
-
 void         file_data_unref             (FileData         *fd);
-
 void         file_data_set_path          (FileData         *fd,
 					  const char       *path);
-
 void         file_data_update            (FileData         *fd);
-
 void         file_data_update_comment    (FileData         *fd);
-
+GList*       file_data_list_dup          (GList            *list);
 void         file_data_list_free         (GList            *list);
 
+const char * file_data_local_path        (FileData         *fd);
+
 #endif /* FILE_DATA_H */
