@@ -44,6 +44,7 @@
 
 #include "gthumb-init.h"
 #include "gthumb-error.h"
+#include "glib-utils.h"
 #include "gconf-utils.h"
 #include "file-utils.h"
 
@@ -302,6 +303,9 @@ path_list_dup (GList *path_list)
 	GList *new_list = NULL;
 	GList *scan;
 
+	if (path_list == NULL)
+		return new_list;
+
 	for (scan = path_list; scan; scan = scan->next)
 		new_list = g_list_prepend (new_list, g_strdup (scan->data));
 
@@ -336,7 +340,7 @@ path_list_find_path (GList *list, const char *path)
 	GList *scan;
 
 	for (scan = list; scan; scan = scan->next)
-		if (strcmp ((char*) scan->data, path) == 0)
+		if (same_uri ((char*) scan->data, path))
 			return scan;
 	return NULL;
 }
@@ -589,7 +593,7 @@ get_extension (const char *path)
 		return NULL;
 
 	p = len - 1;
-	while ((ptr[p] != '.') && (p >= 0)) 
+	while ((p >= 0) && (ptr[p] != '.'))
 		p--;
 
 	if (p < 0)
@@ -622,6 +626,20 @@ get_mime_type (const char *uri)
 }
 
 
+const char *
+get_mime_type_from_ext (const char *ext)
+{
+	char       *filename;
+	const char *result;
+
+	filename = g_strconcat ("x.", ext, NULL);
+	result = get_mime_type (filename);
+	g_free (filename);
+
+	return result;
+}
+
+
 gboolean
 file_is_image (const gchar *name,
 	       gboolean     fast_file_type)
@@ -644,7 +662,7 @@ file_is_image (const gchar *name,
 
 			if (n3 != NULL)
 				result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
-
+			
 			g_free (n3);
 			g_free (n2);
 			g_free (n1);
@@ -687,7 +705,7 @@ xfer_file (const char *from,
 	GnomeVFSXferOptions  opt;
 	GnomeVFSResult       result;
 
-	if (strcmp (from, to) == 0) {
+	if (same_uri (from, to)) {
 		g_warning ("cannot copy file %s: source and destination are the same\n", from);
 		return FALSE;
 	}
@@ -726,6 +744,24 @@ file_move (const gchar *from,
 }
 
 
+gboolean
+file_rename (const gchar *old_path,
+	     const gchar *new_path)
+{
+	GnomeVFSResult  r;
+	char           *e_old_path;
+	char           *e_new_path;
+
+	e_old_path = escape_uri (old_path);
+	e_new_path = escape_uri (new_path);
+	r = gnome_vfs_move (e_old_path, e_new_path, TRUE);
+	g_free (e_old_path);
+	g_free (e_new_path);
+
+	return (r == GNOME_VFS_OK);
+}
+
+
 gboolean
 file_unlink (const gchar *path)
 {
@@ -740,10 +776,9 @@ file_unlink (const gchar *path)
 }
 
 
-static gboolean 
-image_is_type__common (const char *name,
-		       const char *type,
-		       gboolean    fast_file_type)
+const char*
+get_file_mime_type (const char *name,
+		    gboolean    fast_file_type)
 {
 	const char *result = NULL;
 
@@ -761,11 +796,17 @@ image_is_type__common (const char *name,
 	} else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
 	
-	/* Unknown file type. */
-	if (result == NULL)
-		return FALSE;
+	return result;
+}
 
-	return (strcmp (result, type) == 0);
+
+static gboolean 
+image_is_type__common (const char *name, 
+		       const char *type,
+		       gboolean    fast_file_type)
+{
+	const char *result = get_file_mime_type (name, fast_file_type);  
+	return (strcmp_null_tollerant (result, type) == 0);
 }
 
 
@@ -1003,6 +1044,8 @@ get_home_uri (void)
 const char *
 get_file_path_from_uri (const char *uri)
 {
+	if (uri == NULL)
+		return NULL;
 	if (uri_scheme_is_file (uri))
 		return uri + FILE_PREFIX_L;
 	else if (uri[0] == '/')
@@ -1118,14 +1161,16 @@ get_uri_display_name (const char *uri)
 	/* if it is a catalog then remove the extension */
 	
 	catalog_or_search = uri_scheme_is_catalog (uri) || uri_scheme_is_search (uri);
-	if (catalog_or_search)
+	if (catalog_or_search && file_extension_is (uri, CATALOG_EXT))
 		tmp_path[strlen (tmp_path) - strlen (CATALOG_EXT)] = 0;
 			 
-	if ((tmp_path == NULL) 
-	    || (strcmp (tmp_path, "") == 0)
-	    || (strcmp (tmp_path, "/") == 0))
-		name = g_strdup (_("File System"));
-	else {
+	if ((tmp_path == NULL) || (strcmp (tmp_path, "") == 0)
+	    || (strcmp (tmp_path, "/") == 0)) {
+	    	if (catalog_or_search)
+	    		name = g_strdup (_("Catalogs"));
+	    	else
+			name = g_strdup (_("File System"));
+	} else {
 		if (catalog_or_search) {
 			char *base_uri;
 			int   base_uri_len;
@@ -1208,6 +1253,33 @@ path_in_path (const char  *path_src,
 }
 
 
+int
+uricmp (const char *path1,
+	const char *path2)
+{
+	char *uri1, *uri2;
+	int   result;
+
+	uri1 = get_uri_from_path (path1);
+	uri2 = get_uri_from_path (path2);
+
+	result = strcmp_null_tollerant (uri1, uri2);
+
+	g_free (uri1);
+	g_free (uri2);
+
+	return result;
+}
+
+
+gboolean
+same_uri (const char *uri1,
+	  const char *uri2)
+{
+	return uricmp (uri1, uri2) == 0;
+}
+
+
 /* example 1 : filename = /xxx/yyy/zzz/foo 
  *             destdir  = /xxx/www 
  *             return   : ../yyy/zzz/foo
@@ -1217,7 +1289,7 @@ path_in_path (const char  *path_src,
  *             return   : yyy/foo
  */
 char *
-get_path_relative_to_dir (const char *filename, 
+get_path_relative_to_dir (const char *uri, 
 			  const char *destdir)
 {
 	char     *sourcedir;
@@ -1227,9 +1299,9 @@ get_path_relative_to_dir (const char *filename,
 	char     *result;
 	GString  *relpath;
 
-	sourcedir = remove_level_from_path (filename);
+	sourcedir = remove_level_from_path (remove_scheme_from_uri (uri));
 	sourcedir_v = g_strsplit (sourcedir, "/", 0);
-	destdir_v = g_strsplit (destdir, "/", 0);
+	destdir_v = g_strsplit (remove_scheme_from_uri (destdir), "/", 0);
 
 	relpath = g_string_new (NULL);
 
@@ -1250,7 +1322,7 @@ get_path_relative_to_dir (const char *filename,
 		j++;
 	}
 
-	g_string_append (relpath, file_name_from_path (filename));
+	g_string_append (relpath, file_name_from_path (uri));
 
 	g_strfreev (sourcedir_v);
 	g_strfreev (destdir_v);
@@ -1505,6 +1577,29 @@ resolve_all_symlinks (const char  *text_uri,
 }
 
 
+gboolean
+uri_is_root (const char *uri)
+{
+	int len;
+	
+	if (uri == NULL)
+		return FALSE;
+		
+	if (strcmp (uri, "/") == 0)
+		return TRUE;
+		
+	len = strlen (uri);
+	if (strncmp (uri + len - 3, "://", 3) == 0)
+		return TRUE;
+	if (strncmp (uri + len - 2, ":/", 2) == 0)
+		return TRUE;
+	if (strncmp (uri + len - 1, ":", 1) == 0)
+		return TRUE;
+	
+	return FALSE;
+}
+
+
 /* Catalogs */
 
 
@@ -1803,7 +1898,7 @@ remove_extension_from_path (const char *path)
 		return g_strdup (path);
 
 	p = len - 1;
-	while ((ptr[p] != '.') && (p > 0)) 
+	while ((p > 0) && (ptr[p] != '.')) 
 		p--;
 	if (p == 0) 
 		p = len;
@@ -1815,42 +1910,48 @@ remove_extension_from_path (const char *path)
 
 /* temp */
 
-
+/* NOTE: the directory created by this function must deleted when no longer 
+needed. (delete all temporary files and use dir_remove() */
+/* NOTE: the pointer returned by this function must be freed using g_free() */
 char *
 get_temp_dir_name (void)
 {
-	static int  count = 0;
-	char       *tmp_dir = NULL;
+	char       *tmppath;
+	char       *retval;
 	
-	do {
-		g_free (tmp_dir);
-		tmp_dir = g_strdup_printf ("%s%s.%d.%d",
-					   g_get_tmp_dir (),
-					   "/gthumb",
-					   getpid (),
-					   count++);
+	tmppath = (char*) g_build_filename (g_get_tmp_dir (), "gthumb.XXXXXX", NULL);
+	
+	retval = mkdtemp(tmppath);
+	if (retval == NULL)
+		g_free(tmppath);
 
-	} while (path_is_dir (tmp_dir));
-
-	if (! dir_make (tmp_dir, 0700)) {
-		g_free (tmp_dir);
-		tmp_dir = NULL;
-	}
-
-	return tmp_dir;
+	return retval;
 }
 
 
+/* WARNING: this is not secure unless only the current user has write access to 
+tmpdir.  Use get_temp_dir_name to create tmpdir securely. */
+/* NOTE: the pointer returned by this function must be freed using g_free() */
 char *
-get_temp_file_name (const char *ext)
+get_temp_file_name (const char* tmpdir, const char *ext)
 {
-	char *dir, *name, *filename;
+	char *name, *filename;
+	static GStaticMutex count_mutex = G_STATIC_MUTEX_INIT;
+	static int count = 0;
+	
+	if (tmpdir == NULL)
+		return NULL;
+
+	g_static_mutex_lock(&count_mutex);
+	if (ext != NULL)
+		name = g_strdup_printf("%d%s", count++, ext);
+	else
+		name = g_strdup_printf("%d", count++);
+	g_static_mutex_unlock(&count_mutex);
+	
+	filename = g_build_filename (tmpdir, name, NULL);
 
-	dir = get_temp_dir_name ();
-	name = g_strconcat ("temp", ext, NULL);
-	filename = g_build_filename (dir, name, NULL);
 	g_free (name);
-	g_free (dir);
 	
 	return filename;
 }
