@@ -108,6 +108,8 @@ gboolean            file_copy                     (const char       *from,
 						   const char       *to);
 gboolean            file_move                     (const char       *from, 
 						   const char       *to);
+gboolean            file_rename                   (const gchar      *old_path,
+						   const gchar      *new_path);
 gboolean            file_unlink                   (const char       *path);
 gboolean            image_is_jpeg                 (const char       *name);
 gboolean            image_is_gif                  (const char       *name);
@@ -138,6 +140,10 @@ char *              get_uri_display_name          (const char       *uri);
 G_CONST_RETURN char*file_name_from_path           (const char       *path);
 gboolean            path_in_path                  (const char       *path_src,
 						   const char       *path_dest);
+int                 uricmp                        (const char       *uri1,
+						   const char       *uri2);
+gboolean            same_uri                      (const char       *uri1,
+						   const char       *uri2);
 
 char *              get_path_relative_to_dir      (const char       *filename, 
 						   const char       *destdir);
@@ -150,6 +156,7 @@ GnomeVFSURI *       new_uri_from_path             (const char       *path);
 char *              new_path_from_uri             (GnomeVFSURI      *uri);
 GnomeVFSResult      resolve_all_symlinks          (const char       *text_uri,
 						   char            **resolved_text_uri);
+gboolean            uri_is_root                   (const char       *uri);
 
 /* Catalogs */
 
@@ -177,7 +184,8 @@ char *              remove_extension_from_path    (const char       *path);
 /* temp */
 
 char *              get_temp_dir_name             (void);
-char *              get_temp_file_name            (const char       *ext);
+char *              get_temp_file_name            (const char       *tmpdir,
+						   const char       *ext);
 
 /* VFS extensions */
 
@@ -190,6 +198,9 @@ GnomeVFSResult      _gnome_vfs_write_line         (GnomeVFSHandle   *handle,
 						   ...);
 GnomeVFSFileSize    get_dest_free_space           (const char       *path);
 const char *        get_mime_type                 (const char       *path);
+const char*         get_file_mime_type            (const char       *path,
+						   gboolean          fast_file_type);
+const char *        get_mime_type_from_ext        (const char       *ext);
 gboolean            is_mime_type_writable         (const char       *mime_type);
 gboolean            check_permissions             (const char       *path, 
 						   int               mode);
