@@ -222,7 +222,10 @@ _g_substitute_pattern (const char *utf8_text,
 
 			ch = g_utf8_get_char (s);
 			if (ch == pattern)
-				g_string_append (r, value);
+				{
+				if (value)
+					g_string_append (r, value);
+				}
 			else {
 				g_string_append (r, "%");
 				g_string_append_unichar (r, ch);
@@ -393,14 +396,14 @@ get_file_list_from_url_list (char *url_list)
 int
 strcmp_null_tollerant (const char *s1, const char *s2)
 {
-	if ((s1 == NULL) && (s2 == NULL))
-		return 0;
-	else if ((s1 != NULL) && (s2 == NULL))
-		return 1;
-	else if ((s1 == NULL) && (s2 != NULL))
-		return -1;
-	else 
-		return strcmp (s1, s2);
+        if ((s1 == NULL) && (s2 == NULL))
+                return 0;
+        else if (s2 == NULL)
+                return 1;
+        else if (s1 == NULL)
+                return -1;
+        else
+                return strcmp (s1, s2);
 }
 
 
