@@ -38,6 +38,7 @@
 #include "file-data.h"
 #include "icons/pixbufs.h"
 #include "pixbuf-utils.h"
+#include "gth-sort-utils.h"
 
 
 enum {
@@ -1043,6 +1044,7 @@ gfv_sorted (GthFileView   *file_view,
 	case GTH_SORT_METHOD_BY_PATH: col = COLUMN_PATH; break;
 	case GTH_SORT_METHOD_BY_SIZE: col = COLUMN_SIZE; break;
 	case GTH_SORT_METHOD_BY_TIME: col = COLUMN_TIME; break;
+	case GTH_SORT_METHOD_BY_COMMENT: col = COLUMN_COMMENT; break;
 	default: col = GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID; break;
 	}
 
@@ -1476,7 +1478,8 @@ add_columns (GtkTreeView *treeview)
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
 	int                i, j;
-
+	GValue             value = { 0, };
+	
 	/* The Name column. */
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (column, _("Name"));
@@ -1488,6 +1491,11 @@ add_columns (GtkTreeView *treeview)
                                              NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
+        g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+        g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+        g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+        g_value_unset (&value);
+        	
 	gtk_tree_view_column_pack_start (column,
 					 renderer,
 					 TRUE);
@@ -1495,7 +1503,7 @@ add_columns (GtkTreeView *treeview)
                                              "text", COLUMN_NAME,
                                              NULL);
 
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	/*gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);*/
 	/*
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME);
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_PATH);
@@ -1532,7 +1540,7 @@ comp_func_name (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return strcasecmp (fd1->name, fd2->name);
+	return gth_sort_by_filename_but_ignore_path (fd1->name, fd2->name);
 }
 
 
@@ -1545,11 +1553,7 @@ comp_func_size (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->size < fd2->size) return -1;
-	if (fd1->size > fd2->size) return 1;
-
-	/* if the size is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+	return gth_sort_by_size_then_name (fd1->size, fd2->size, fd1->path, fd2->path);
 }
 
 
@@ -1562,11 +1566,21 @@ comp_func_time (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->mtime < fd2->mtime) return -1;
-	if (fd1->mtime > fd2->mtime) return 1;
+	return gth_sort_by_filetime_then_name (fd1->mtime, fd2->mtime,
+						 fd1->path, fd2->path);
+}
 
-	/* if time is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+
+static int
+comp_func_exif_date (gconstpointer  ptr1,
+		     gconstpointer  ptr2)
+{
+	FileData *fd1 = (FileData *) ptr1, *fd2 = (FileData *) ptr2;
+	
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_exiftime_then_name (fd1, fd2);
 }
 
 
@@ -1579,7 +1593,17 @@ comp_func_path (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return uricmp (fd1->path, fd2->path);
+	return gth_sort_by_full_path (fd1->path, fd2->path);
+}
+
+
+static int
+comp_func_comment (gconstpointer ptr1, gconstpointer ptr2)
+{
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
+
+	return gth_sort_by_comment_then_name (fd1->comment, fd2->comment,
+					      fd1->path, fd2->path);
 }
 
 
@@ -1609,6 +1633,12 @@ get_compfunc_from_method (GthSortMethod sort_method)
 	case GTH_SORT_METHOD_BY_PATH:
 		func = comp_func_path;
 		break;
+	case GTH_SORT_METHOD_BY_COMMENT:
+		func = comp_func_comment;
+		break;
+	case GTH_SORT_METHOD_BY_EXIF_DATE:
+		func = comp_func_exif_date;	
+		break;	
 	default:
 		func = comp_func_none;
 		break;
