@@ -32,6 +32,7 @@
 #include "file-data.h"
 #include "icons/pixbufs.h"
 #include "pixbuf-utils.h"
+#include "gth-sort-utils.h"
 
 
 struct _GthFileViewThumbsPrivate {
@@ -591,6 +592,15 @@ gfv_get_last_visible (GthFileView *file_view)
 }
 
 
+static void
+gfv_set_visible_func (GthFileView    *file_view,
+                      GthVisibleFunc  func,
+                      gpointer        data)
+{
+	/* FIXME */
+}
+
+
 /* Sort */
 
 
@@ -607,7 +617,7 @@ comp_func_name (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return strcasecmp (fd1->name, fd2->name);
+	return gth_sort_by_filename_but_ignore_path (fd1->name, fd2->name);
 }
 
 
@@ -624,11 +634,7 @@ comp_func_size (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->size < fd2->size) return -1;
-	if (fd1->size > fd2->size) return 1;
-
-	/* if the size is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+	return gth_sort_by_size_then_name (fd1->size, fd2->size, fd1->path, fd2->path);
 }
 
 
@@ -645,11 +651,25 @@ comp_func_time (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->mtime < fd2->mtime) return -1;
-	if (fd1->mtime > fd2->mtime) return 1;
+	return gth_sort_by_filetime_then_name (fd1->mtime, fd2->mtime,
+						 fd1->path, fd2->path);
+}
 
-	/* if time is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+
+static int
+comp_func_exif_date (gconstpointer  ptr1,
+		     gconstpointer  ptr2)
+{
+	const GthImageListItem *item1 = ptr1, *item2 = ptr2;
+	FileData               *fd1, *fd2;
+
+	fd1 = item1->data;
+	fd2 = item2->data;
+
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_exiftime_then_name (fd1, fd2);
 }
 
 
@@ -666,7 +686,21 @@ comp_func_path (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return uricmp (fd1->path, fd2->path);
+	return gth_sort_by_full_path (fd1->path, fd2->path);
+}
+
+
+static int
+comp_func_comment (gconstpointer  ptr1, gconstpointer  ptr2)
+{
+	const GthImageListItem *item1 = ptr1, *item2 = ptr2;
+	const FileData         *fd1, *fd2;
+
+	fd1 = item1->data;
+	fd2 = item2->data;
+
+	return gth_sort_by_comment_then_name (item1->comment, item2->comment,
+					fd1->path, fd2->path);
 }
 
 
@@ -696,6 +730,12 @@ get_compfunc_from_method (GthSortMethod sort_method)
 	case GTH_SORT_METHOD_BY_PATH:
 		func = comp_func_path;
 		break;
+	case GTH_SORT_METHOD_BY_COMMENT:
+		func = comp_func_comment;
+		break;
+	case GTH_SORT_METHOD_BY_EXIF_DATE:
+		func = comp_func_exif_date;	
+		break;	
 	case GTH_SORT_METHOD_NONE:
 	case GTH_SORT_METHOD_MANUAL:
 	default:
@@ -1015,6 +1055,7 @@ gth_file_view_thumbs_class_init (GthFileViewThumbsClass *file_view_thumbs_class)
 	file_view_class->get_image_at         = gfv_get_image_at;
 	file_view_class->get_first_visible    = gfv_get_first_visible;
 	file_view_class->get_last_visible     = gfv_get_last_visible;
+	file_view_class->set_visible_func     = gfv_set_visible_func;
 	file_view_class->sorted               = gfv_sorted;
 	file_view_class->unsorted             = gfv_unsorted;
 	file_view_class->image_activated      = gfv_image_activated;
