@@ -64,6 +64,7 @@
 #define  PREF_ZOOM_QUALITY           "/apps/gthumb/viewer/zoom_quality"
 #define  PREF_ZOOM_CHANGE            "/apps/gthumb/viewer/zoom_change"
 #define  PREF_TRANSP_TYPE            "/apps/gthumb/viewer/transparency_type"
+#define  PREF_RESET_SCROLLBARS       "/apps/gthumb/viewer/reset_scrollbars"
 #define  PREF_CHECK_TYPE             "/apps/gthumb/viewer/check_type"
 #define  PREF_CHECK_SIZE             "/apps/gthumb/viewer/check_size"
 #define  PREF_BLACK_BACKGROUND       "/apps/gthumb/viewer/black_background"
@@ -163,6 +164,7 @@
 #define  PREF_WEB_ALBUM_RESIZE_HEIGHT "/apps/gthumb/dialogs/web_album/resize_height"
 #define  PREF_WEB_ALBUM_ROWS          "/apps/gthumb/dialogs/web_album/rows"
 #define  PREF_WEB_ALBUM_COLUMNS       "/apps/gthumb/dialogs/web_album/columns"
+#define  PREF_WEB_ALBUM_SINGLE_INDEX  "/apps/gthumb/dialogs/web_album/single_index"
 #define  PREF_WEB_ALBUM_SORT          "/apps/gthumb/dialogs/web_album/sort_by"
 #define  PREF_WEB_ALBUM_REVERSE       "/apps/gthumb/dialogs/web_album/reverse_order"
 #define  PREF_WEB_ALBUM_HEADER        "/apps/gthumb/dialogs/web_album/header"
@@ -174,8 +176,9 @@
 #define  PREF_SEARCH_RECURSIVE        "/apps/gthumb/dialogs/search/recursive"
 
 #define  PREF_PRINT_PAPER_SIZE        "/apps/gthumb/dialogs/print/paper_size"
-#define  PREF_PRINT_PAPER_ORIENTATION "/apps/gthumb/dialogs/print/paper_orientation"
+#define  PREF_PRINT_PAPER_ORIENTATION "/apps/gthumb/dialogs/print/print_paper_orientation"
 #define  PREF_PRINT_INCLUDE_COMMENT   "/apps/gthumb/dialogs/print/include_comment"
+#define  PREF_PRINT_INCLUDE_FILENAME  "/apps/gthumb/dialogs/print/include_filename"
 #define  PREF_PRINT_COMMENT_FONT      "/apps/gthumb/dialogs/print/comment_font"
 #define  PREF_PRINT_PAPER_WIDTH       "/apps/gthumb/dialogs/print/paper_width"
 #define  PREF_PRINT_PAPER_HEIGHT      "/apps/gthumb/dialogs/print/paper_height"
@@ -195,7 +198,7 @@
 #define  PREF_PHOTO_IMPORT_FILM           "/apps/gthumb/dialogs/photo_importer/film"
 #define  PREF_PHOTO_IMPORT_DELETE         "/apps/gthumb/dialogs/photo_importer/delete_from_camera"
 #define  PREF_PHOTO_IMPORT_KEEP_FILENAMES "/apps/gthumb/dialogs/photo_importer/keep_original_filenames"
-#define  PREF_PHOTO_IMPORT_ADJUST_ORIENTATION "/apps/gthumb/dialogs/photo_importer/adjust_orientation"
+#define  PREF_PHOTO_IMPORT_RESET_EXIF_ORIENTATION	"/apps/gthumb/dialogs/photo_importer/reset_exif_orientation_on_import"
 #define  PREF_PHOTO_IMPORT_MODEL          "/apps/gthumb/dialogs/photo_importer/model"
 #define  PREF_PHOTO_IMPORT_PORT           "/apps/gthumb/dialogs/photo_importer/port"
 
@@ -205,6 +208,8 @@
 #define  PREF_ADD_TO_CATALOG_LAST_CATALOG "/apps/gthumb/dialogs/add_to_catalog/last_catalog"
 #define  PREF_ADD_TO_CATALOG_VIEW         "/apps/gthumb/dialogs/add_to_catalog/view"
 
+#define  PREF_ROTATE_RESET_EXIF_ORIENTATION     "/apps/gthumb/dialogs/rotate/reset_exif_orientation_on_rotate"
+
 typedef struct {
 	Bookmarks   *bookmarks;
 
