@@ -93,8 +93,9 @@ static void
 add_columns (CatalogList *cat_list,
 	     GtkTreeView *treeview)
 {
-	GtkCellRenderer *renderer;
+	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
+	GValue             value = { 0, };
 	
 	/* The Name column. */
 
@@ -107,6 +108,11 @@ add_columns (CatalogList *cat_list,
 					     NULL);
 	
 	cat_list->text_renderer = renderer = gtk_cell_renderer_text_new ();
+	
+	g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+        g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+        g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+        g_value_unset (&value);
 
         gtk_tree_view_column_pack_start (column,
                                          renderer,
@@ -440,7 +446,7 @@ catalog_list_get_iter_from_path (CatalogList *cat_list,
 
 		g_return_val_if_fail (row_path != NULL, FALSE);
 
-		if (strcmp (row_path, path) == 0) {
+		if (same_uri (row_path, path)) {
 			g_free (row_path);
 			return TRUE;
 		}
@@ -457,12 +463,15 @@ catalog_list_get_iter_from_path (CatalogList *cat_list,
 void
 catalog_list_update_click_policy (CatalogList *cat_list)
 {
-	GdkWindow  *win = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (cat_list->list_view));
+	GdkWindow  *win;
 	GdkDisplay *display;
 
 	cat_list->single_click = (pref_get_real_click_policy () == GTH_CLICK_POLICY_SINGLE);
 
-	gdk_window_set_cursor (win, NULL);
+	win = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (cat_list->list_view));
+	if (win != NULL)
+		gdk_window_set_cursor (win, NULL);
+
 	display = gtk_widget_get_display (GTK_WIDGET (cat_list->list_view));
 	if (display != NULL) 
 		gdk_display_flush (display);
@@ -510,7 +519,7 @@ catalog_list_refresh (CatalogList *cat_list)
 	/* Add a ".." entry if the current dir is not the base catalog dir. */
 
 	base = get_catalog_full_path (NULL);
-	if (strcmp (base, cat_list->path) != 0) {
+	if (!same_uri (base, cat_list->path)) {
 		char *prev_dir = remove_level_from_path (cat_list->path);
 
 		dir_list = g_list_prepend (dir_list, prev_dir);
@@ -626,7 +635,7 @@ void
 catalog_list_select_iter (CatalogList *cat_list,
 			  GtkTreeIter *iter)
 {
-	GtkTreeView *tree_view;
+	GtkTreeView      *tree_view;
 	GtkTreeSelection *selection;
 
 	tree_view = GTK_TREE_VIEW (cat_list->list_view);
