@@ -35,6 +35,7 @@
 #include "preferences.h"
 #include "main.h"
 #include "glib-utils.h"
+#include "gth-utils.h"
 #include "gth-window.h"
 #include "gtk-utils.h"
 #include "gth-file-view.h"
@@ -74,7 +75,7 @@ typedef struct {
 	GtkWidget     *date_optionmenu;
 	GtkWidget     *date_dateedit;
 	GtkWidget     *save_changed_checkbutton;
-	GtkWidget     *save_button;
+	GtkWidget     *ok_button;
 
 	CommentData   *original_cdata;
 	gboolean       have_exif_data;
@@ -148,9 +149,7 @@ get_requested_time (DialogData *data,
 		t = time (NULL);
 		break;
 	case EXIF_DATE:
-#ifdef HAVE_LIBEXIF
 		t = get_exif_time (filename);
-#endif
 		break;
 	case LAST_MODIFIED_DATE:
 		t = get_file_mtime (filename);
@@ -176,7 +175,6 @@ save_clicked_cb (GtkWidget  *widget,
 	struct tm    tm = {0};
 	GtkTextIter  start_iter, end_iter;
 	gboolean     save_changed;
-	gboolean     close_on_save;
 
 	save_changed = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->save_changed_checkbutton));
 	cdata = comment_data_new ();
@@ -269,11 +267,7 @@ save_clicked_cb (GtkWidget  *widget,
 
 	/**/
 
-	g_object_get (data->window,
-		      "close_on_save", &close_on_save,
-		      NULL);
-	if (close_on_save)
-		dlg_comment_close (data->dialog);
+	dlg_comment_close (data->dialog);
 }
 
 
@@ -282,31 +276,7 @@ static void
 help_cb (GtkWidget  *widget, 
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display ("gthumb", "gthumb-comments", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-comments");
 }
 
 
@@ -338,22 +308,10 @@ date_optionmenu_changed_cb (GtkOptionMenu *option_menu,
 					  get_file_mtime (first_image));
 		break;
 	case EXIF_DATE:
-#ifdef HAVE_LIBEXIF
 		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit),
 					  get_exif_time (first_image));
-#endif
 		break;
 	}
-
-	gtk_widget_set_sensitive (data->save_button, TRUE);
-}
-
-
-static void
-editabled_changed_cb (GtkEditable *editable,
-		      DialogData  *data)
-{
-	gtk_widget_set_sensitive (data->save_button, TRUE);
 }
 
 
@@ -412,8 +370,8 @@ dlg_comment_new (GthWindow *window)
 	data->date_optionmenu = glade_xml_get_widget (data->gui, "date_optionmenu");
 	data->date_dateedit = glade_xml_get_widget (data->gui, "date_dateedit");
 	data->save_changed_checkbutton = glade_xml_get_widget (data->gui, "save_changed_checkbutton");
-	data->save_button = glade_xml_get_widget (data->gui, "save_button");
-	btn_close = glade_xml_get_widget (data->gui, "close_button");
+	data->ok_button = glade_xml_get_widget (data->gui, "ok_button");
+	btn_close = glade_xml_get_widget (data->gui, "cancel_button");
 	btn_help = glade_xml_get_widget (data->gui, "help_button");
 
 	data->note_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data->note_text_view));
@@ -430,7 +388,7 @@ dlg_comment_new (GthWindow *window)
 			  "unrealize",
 			  G_CALLBACK (unrealize_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->save_button), 
+	g_signal_connect (G_OBJECT (data->ok_button), 
 			  "clicked",
 			  G_CALLBACK (save_clicked_cb),
 			  data);
@@ -448,15 +406,6 @@ dlg_comment_new (GthWindow *window)
 			  G_CALLBACK (date_optionmenu_changed_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (data->place_entry), 
-			  "changed",
-			  G_CALLBACK (editabled_changed_cb),
-			  data);
-	g_signal_connect (G_OBJECT (data->note_text_buffer), 
-			  "changed",
-			  G_CALLBACK (editabled_changed_cb),
-			  data);
-
 	/* run dialog. */
 
 	gtk_widget_grab_focus (data->note_text_view);
@@ -491,7 +440,7 @@ dlg_comment_update (GtkWidget *dlg)
 	data->file_list = gth_window_get_file_list_selection (data->window); 
 
 	if (data->file_list == NULL) {
-		gtk_widget_set_sensitive (data->save_button, FALSE);
+		gtk_widget_set_sensitive (data->ok_button, FALSE);
 		gtk_widget_set_sensitive (data->comment_main_box, FALSE);
 		return;
 	} else 
@@ -515,14 +464,7 @@ dlg_comment_update (GtkWidget *dlg)
 
 	/**/
 
-#ifndef HAVE_LIBEXIF
-	data->have_exif_data = FALSE;
-#else
-	{
-		char *first_image = data->file_list->data;
-		data->have_exif_data = have_exif_time (first_image);
-	}
-#endif
+	data->have_exif_data = have_exif_time (data->file_list->data);
 	
 	first_image = data->file_list->data;
 	data->original_cdata = cdata = comments_load_comment (first_image, TRUE);
@@ -605,8 +547,6 @@ dlg_comment_update (GtkWidget *dlg)
 		gtk_option_menu_set_history (GTK_OPTION_MENU (data->date_optionmenu), NO_DATE);
 		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), ctime);
 	}
-
-	gtk_widget_set_sensitive (data->save_button, FALSE);
 }
 
 
