@@ -24,13 +24,18 @@
 #include <sys/types.h>
 #include <string.h>
 #include <unistd.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomeui/gnome-help.h>
 #include <glade/glade.h>
+
 #include "file-data.h"
 #include "gconf-utils.h"
-#include "gthumb-window.h"
+#include "gth-browser.h"
+#include "gth-utils.h"
 #include "gtk-utils.h"
 #include "image-viewer.h"
 #include "main.h"
@@ -48,7 +53,7 @@
 
 
 typedef struct {
-	GThumbWindow *window;
+	GthWindow    *window;
 	GladeXML     *gui;
 	GladeXML     *progress_gui;
 
@@ -57,7 +62,8 @@ typedef struct {
 	GtkWidget    *conv_png_radiobutton;
 	GtkWidget    *conv_tga_radiobutton;
 	GtkWidget    *conv_tiff_radiobutton;
-	GtkWidget    *conv_om_optionmenu;
+	GtkWidget    *conv_dest_filechooserbutton;
+	GtkWidget    *conv_om_combobox;
 	GtkWidget    *conv_remove_orig_checkbutton;
 
 	GtkWidget    *rename_dialog;
@@ -83,6 +89,7 @@ typedef struct {
 	GdkPixbuf    *pixbuf;
 	const char   *image_type;
 	const char   *ext;
+	char         *destination;
 	char         *new_path;	
 	char        **keys;
 	char        **values;
@@ -108,6 +115,7 @@ destroy_cb (GtkWidget  *widget,
 	g_strfreev (data->values);	
 
 	g_free (data->new_path);
+	g_free (data->destination);
 
 	g_object_unref (data->gui);
 	g_object_unref (data->progress_gui);
@@ -128,21 +136,27 @@ load_next_image (DialogData *data)
 }
 
 
+static void
+stop_operation (DialogData *data)
+{
+	all_windows_notify_files_created (data->saved_list);
+	all_windows_notify_files_deleted (data->deleted_list);
+	all_windows_add_monitor ();
+	gtk_widget_destroy (data->dialog);
+}
+
+
 static void
 load_current_image (DialogData *data)
 {
 	FileData  *fd;
-	char      *folder;
 	char      *name_no_ext;
 	char      *utf8_name;
 	char      *message;
 
 	if (data->stop_convertion || (data->current_image == NULL)) {
-		all_windows_notify_files_created (data->saved_list);
-		all_windows_notify_files_deleted (data->deleted_list);
-		all_windows_add_monitor ();
 		gtk_widget_destroy (data->progress_dialog);
-		gtk_widget_destroy (data->dialog);
+		stop_operation (data);
 		return;
 	}
 
@@ -150,15 +164,13 @@ load_current_image (DialogData *data)
 	data->new_path = NULL;
 
 	fd = (FileData*) data->current_image->data;
-	folder = remove_level_from_path (fd->path);
 	name_no_ext = remove_extension_from_path (file_name_from_path (fd->path));
 
-	data->new_path = g_strconcat (folder, "/", name_no_ext, ".", data->ext, 0);
+	data->new_path = g_strconcat (data->destination, "/", name_no_ext, ".", data->ext, NULL);
 
-	g_free (folder);
 	g_free (name_no_ext);
 
-	utf8_name = g_filename_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
+	utf8_name = g_filename_display_basename (data->new_path);
 	message = g_strdup_printf (_("Converting image: %s"), utf8_name);
 
 	gtk_label_set_text (GTK_LABEL (data->progress_label), message);
@@ -167,7 +179,7 @@ load_current_image (DialogData *data)
 	g_free (message);
 
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (data->progress_bar),
-				       (double) data->image / data->images);
+				       (double) data->image / (data->images+1));
 	
 	image_loader_set_path (data->loader, fd->path);
 	image_loader_start (data->loader);
@@ -180,7 +192,7 @@ show_rename_dialog (DialogData *data)
 	char  *message;
 	char  *utf8_name;
 
-	utf8_name = g_filename_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
+	utf8_name = g_filename_display_basename (data->new_path);
 
 	message = g_strdup_printf (_("An image named \"%s\" is already present. " "Please specify a different name."), utf8_name);
 	
@@ -191,6 +203,7 @@ show_rename_dialog (DialogData *data)
 
 	g_free (utf8_name);
 
+	gtk_window_set_modal (GTK_WINDOW (data->rename_dialog), TRUE);
 	gtk_widget_show (data->rename_dialog);
 	gtk_widget_grab_focus (data->conv_ren_name_entry);
 }
@@ -202,7 +215,7 @@ save_image_and_remove_original (DialogData *data)
 	GError   *error = NULL;
 
 	if (path_is_file (data->new_path))
-		unlink (data->new_path);
+		file_unlink (data->new_path);
 
 	if (_gdk_pixbuf_savev (data->pixbuf, 
 			       data->new_path, 
@@ -213,9 +226,8 @@ save_image_and_remove_original (DialogData *data)
 		FileData *fd = data->current_image->data;
 		data->saved_list = g_list_prepend (data->saved_list, g_strdup (data->new_path));
 
-		if (data->remove_original 
-		    && (strcmp (fd->path, data->new_path) != 0)) {
-			unlink (fd->path);
+		if (data->remove_original && ! same_uri (fd->path, data->new_path)) {
+			file_unlink (fd->path);
 			data->deleted_list = g_list_prepend (data->deleted_list, g_strdup (fd->path));
 		}
 	} else 
@@ -229,6 +241,7 @@ rename_response_cb (GtkWidget  *dialog,
 		    DialogData *data)
 {
 	gtk_widget_hide (dialog);
+	gtk_window_set_modal (GTK_WINDOW (dialog), FALSE);
 
 	if (response_id == GTK_RESPONSE_OK) {
 		char *new_name, *folder;
@@ -304,7 +317,7 @@ loader_done (ImageLoader *il,
 			break;
 
 		case GTH_OVERWRITE_ASK:
-			utf8_name = g_filename_to_utf8 (file_name_from_path (data->new_path), -1, 0, 0, 0);
+			utf8_name = g_filename_display_basename (data->new_path);
 			message = g_strdup_printf (_("An image named \"%s\" is already present. " "Do you want to overwrite it?"), utf8_name);
 
 			d = _gtk_yesno_dialog_new (GTK_WINDOW (data->dialog),
@@ -351,6 +364,8 @@ static void
 ok_cb (GtkWidget  *widget, 
        DialogData *data)
 {
+	char *esc_path;
+
 	all_windows_remove_monitor ();
 
 	data->loader = IMAGE_LOADER (image_loader_new (NULL, FALSE));
@@ -380,9 +395,15 @@ ok_cb (GtkWidget  *widget,
 	else
 		data->image_type = data->ext = "jpeg";
 
-	data->overwrite_mode = gtk_option_menu_get_history (GTK_OPTION_MENU (data->conv_om_optionmenu));
+	data->overwrite_mode = gtk_combo_box_get_active (GTK_COMBO_BOX (data->conv_om_combobox));
 	data->remove_original = is_active (data->conv_remove_orig_checkbutton);
 
+	/**/
+
+	esc_path = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->conv_dest_filechooserbutton));
+	data->destination = gnome_vfs_unescape_string (esc_path, "");
+	g_free (esc_path);
+
 	/* Save options. */
 
 	eel_gconf_set_string (PREF_CONVERT_IMAGE_TYPE, data->image_type);
@@ -393,7 +414,7 @@ ok_cb (GtkWidget  *widget,
 
 	gtk_widget_hide (data->dialog);
 
-	if (dlg_save_options (GTK_WINDOW (data->dialog), 
+	if (dlg_save_options (GTK_WINDOW (data->window), 
 			      data->image_type, 
 			      &data->keys, 
 			      &data->values)) {
@@ -406,11 +427,12 @@ ok_cb (GtkWidget  *widget,
 }
 
 
-static void
+static gboolean
 stop_convertion_cb (GtkWidget  *widget, 
 		    DialogData *data)
 {
 	data->stop_convertion = TRUE;
+	return FALSE;
 }
 
 
@@ -419,37 +441,33 @@ static void
 help_cb (GtkWidget  *widget, 
 	 DialogData *data)
 {
-	GError *err;
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-convert-format");
+}
 
-	err = NULL;  
-	gnome_help_display ("gthumb", "gthumb-convert-format", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+
+static gboolean
+progress_dlg_delete_event_cb (GtkWidget  *caller, 
+			      GdkEvent   *event, 
+			      DialogData *data)
+{
+	stop_operation (data);
+	return TRUE;
+}
+
+
+static gboolean
+rename_dlg_delete_event_cb (GtkWidget  *caller, 
+			    GdkEvent   *event, 
+			    DialogData *data)
+{
+	return TRUE;
 }
 
 
 void
-dlg_convert (GThumbWindow *window)
+dlg_convert (GthBrowser *browser)
 {
+	GthWindow   *window = GTH_WINDOW (browser);
 	DialogData  *data;
 	GtkWidget   *cancel_button;
 	GtkWidget   *ok_button;
@@ -458,8 +476,9 @@ dlg_convert (GThumbWindow *window)
 	GtkWidget   *progress_cancel;
 	GList       *list;
 	char        *image_type;
+	char        *esc_uri = NULL;
 
-	list = gth_file_list_get_selection_as_fd (window->file_list);
+	list = gth_window_get_file_list_selection_as_fd (window);
 	if (list == NULL) {
 		g_warning ("No file selected.");
 		return;
@@ -469,6 +488,7 @@ dlg_convert (GThumbWindow *window)
 
 	data = g_new0 (DialogData, 1);
 
+	data->window = window;
 	data->file_list = list;
 	data->current_image = list;
 
@@ -502,8 +522,9 @@ dlg_convert (GThumbWindow *window)
 	data->conv_png_radiobutton = glade_xml_get_widget (data->gui, "conv_png_radiobutton");
 	data->conv_tga_radiobutton = glade_xml_get_widget (data->gui, "conv_tga_radiobutton");
 	data->conv_tiff_radiobutton = glade_xml_get_widget (data->gui, "conv_tiff_radiobutton");
-	data->conv_om_optionmenu = glade_xml_get_widget (data->gui, "conv_om_optionmenu");
+	data->conv_om_combobox = glade_xml_get_widget (data->gui, "conv_om_combobox");
 	data->conv_remove_orig_checkbutton = glade_xml_get_widget (data->gui, "conv_remove_orig_checkbutton");
+	data->conv_dest_filechooserbutton = glade_xml_get_widget (data->gui, "conv_dest_filechooserbutton");
 
 	ok_button = glade_xml_get_widget (data->gui, "conv_ok_button");
 	cancel_button = glade_xml_get_widget (data->gui, "conv_cancel_button");
@@ -544,12 +565,27 @@ dlg_convert (GThumbWindow *window)
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (data->conv_om_optionmenu),
-				     pref_get_convert_overwrite_mode ());
+	gtk_combo_box_set_active (GTK_COMBO_BOX (data->conv_om_combobox),
+				  pref_get_convert_overwrite_mode ());
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->conv_remove_orig_checkbutton), 
 				      eel_gconf_get_boolean (PREF_CONVERT_REMOVE_ORIGINAL, FALSE));
 
+	/**/
+
+	esc_uri = gnome_vfs_escape_host_and_path_string (gth_browser_get_current_directory (browser));
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->conv_dest_filechooserbutton), esc_uri);
+	g_free (esc_uri);
+
+	/**/
+
+	gtk_window_set_transient_for (GTK_WINDOW (data->rename_dialog), 
+				      GTK_WINDOW (data->progress_dialog));
+	gtk_window_set_transient_for (GTK_WINDOW (data->progress_dialog), 
+				      GTK_WINDOW (window));
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
+				      GTK_WINDOW (window));
+
 	/* Set the signals handlers. */
 	
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -569,12 +605,18 @@ dlg_convert (GThumbWindow *window)
 			  "clicked",
 			  G_CALLBACK (help_cb),
 			  data);
-
 	g_signal_connect (G_OBJECT (progress_cancel), 
 			  "clicked",
 			  G_CALLBACK (stop_convertion_cb),
 			  data);
-
+	g_signal_connect (G_OBJECT (data->progress_dialog),
+			  "delete_event",
+			  G_CALLBACK (progress_dlg_delete_event_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->rename_dialog),
+			  "delete_event",
+			  G_CALLBACK (rename_dlg_delete_event_cb),
+			  data);
 	g_signal_connect (G_OBJECT (data->rename_dialog), 
 			  "response",
 			  G_CALLBACK (rename_response_cb),
@@ -582,8 +624,6 @@ dlg_convert (GThumbWindow *window)
 
 	/* Run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
-				      GTK_WINDOW (window->app));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE); 
 	gtk_widget_show (data->dialog);
 }
