@@ -33,16 +33,15 @@
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <glade/glade.h>
 
-#ifdef HAVE_LIBEXIF
 #include <libexif/exif-data.h>
 #include <libexif/exif-content.h>
 #include <libexif/exif-entry.h>
 #include <libexif/exif-utils.h>
 #include "jpegutils/jpeg-data.h"
-#endif /* HAVE_LIBEXIF */
 
 #include "file-utils.h"
 #include "gconf-utils.h"
+#include "gth-utils.h"
 #include "gth-window.h"
 #include "gtk-utils.h"
 #include "image-loader.h"
@@ -70,15 +69,14 @@ typedef struct {
 	GtkWidget    *j_revert_button;;
 	GtkWidget    *j_apply_to_all_checkbutton;
 	GtkWidget    *j_preview_image;
-	GtkWidget    *j_from_exif_checkbutton;
+	GtkWidget    *j_reset_exif_tag_on_rotate_checkbutton;
 
-	gboolean      from_exif_data;
 	GList        *file_list;
 	GList        *files_changed_list;
 	GList        *current_image;
 	ImageLoader  *loader;
 	GdkPixbuf    *original_preview;
-	RotationData *rot_data;
+	GthTransform	transform;
 } DialogData;
 
 
@@ -91,21 +89,20 @@ dialog_data_free (DialogData *data)
 		data->files_changed_list = NULL;
 	}
 
-	all_windows_add_monitor ();
+	g_idle_add ((GSourceFunc)all_windows_add_monitor, NULL);
 
 	path_list_free (data->file_list);
 	if (data->loader != NULL)
 		g_object_unref (data->loader);
 	if (data->gui != NULL)
 		g_object_unref (data->gui);
-	g_free (data->rot_data);
 	g_free (data);
 }
 
 
 /* called when the main dialog is closed. */
 static void
-destroy_cb (GtkWidget  *widget, 
+destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	dialog_data_free (data);
@@ -113,7 +110,7 @@ destroy_cb (GtkWidget  *widget,
 
 
 static void
-add_image_to_button (GtkWidget    *button, 
+add_image_to_button (GtkWidget    *button,
 		     const guint8 *rgba)
 {
 	GdkPixbuf *pixbuf;
@@ -127,8 +124,8 @@ add_image_to_button (GtkWidget    *button,
 
 
 static GdkPixbuf *
-_gdk_pixbuf_scale_keep_aspect_ratio (GdkPixbuf *pixbuf, 
-				     int        max_width, 
+_gdk_pixbuf_scale_keep_aspect_ratio (GdkPixbuf *pixbuf,
+				     int        max_width,
 				     int        max_height)
 {
 	int        width, height;
@@ -150,68 +147,7 @@ _gdk_pixbuf_scale_keep_aspect_ratio (GdkPixbuf *pixbuf,
 }
 
 
-
 static void
-update_from_exif_data (DialogData *data)
-{
-#ifdef HAVE_LIBEXIF
-	gboolean   from_exif;
-	GdkPixbuf *src_pixbuf, *dest_pixbuf;
-	
-	from_exif = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton));
-	gtk_widget_set_sensitive (data->j_button_box, !from_exif);
-
-	if (! from_exif)
-		return;
-	
-	update_rotation_from_exif_data (data->current_image->data, data->rot_data);
-
-	src_pixbuf = data->original_preview;
-	if (src_pixbuf == NULL) 
-		return;
-
-	switch (data->rot_data->rot_type) {
-	case GTH_TRANSFORM_ROTATE_90:
-		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
-		break;
-	case GTH_TRANSFORM_ROTATE_180:
-		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, TRUE);
-		break;
-	case GTH_TRANSFORM_ROTATE_270:
-		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
-		break;
-	default:
-		dest_pixbuf = src_pixbuf;
-		g_object_ref (dest_pixbuf);
-		break;
-	}
-
-	src_pixbuf = dest_pixbuf;
-
-	switch (data->rot_data->tran_type) {
-	case GTH_TRANSFORM_MIRROR:
-		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
-		break;
-	case GTH_TRANSFORM_FLIP:
-		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
-		break;
-	default:
-		dest_pixbuf = src_pixbuf;
-		g_object_ref (dest_pixbuf);
-		break;
-	}
-	
-	g_object_unref (src_pixbuf);
-
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
-		
-	g_object_unref (dest_pixbuf);
-
-#endif /* HAVE_LIBEXIF */
-}
-
-
-static void 
 image_loader_done_cb (ImageLoader  *il,
 		      DialogData   *data)
 {
@@ -226,13 +162,11 @@ image_loader_done_cb (ImageLoader  *il,
 
 		gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
 		gtk_widget_set_sensitive (data->j_revert_button, TRUE);
-
-		update_from_exif_data (data);
 	}
 }
 
 
-static void 
+static void
 image_loader_error_cb (ImageLoader  *il,
 		       DialogData   *data)
 {
@@ -250,7 +184,7 @@ load_current_image (DialogData *data)
 		gtk_widget_destroy (data->dialog);
 		return;
 	}
-	
+
 	gtk_widget_set_sensitive (data->j_button_vbox, FALSE);
 	gtk_widget_set_sensitive (data->j_revert_button, FALSE);
 
@@ -258,8 +192,7 @@ load_current_image (DialogData *data)
 	image_loader_set_path (data->loader, path);
 	image_loader_start (data->loader);
 
-	data->rot_data->rot_type = GTH_TRANSFORM_ROTATE_0;
-	data->rot_data->tran_type = GTH_TRANSFORM_NONE;
+	data->transform = GTH_TRANSFORM_NONE;
 }
 
 
@@ -270,7 +203,7 @@ load_next_image (DialogData *data)
 		gtk_widget_destroy (data->dialog);
 		return;
 	}
-	
+
 	data->current_image = data->current_image->next;
 	if (data->current_image == NULL) {
 		gtk_widget_destroy (data->dialog);
@@ -290,11 +223,40 @@ notify_file_changed (DialogData *data,
 		GList *list = g_list_prepend (NULL, (char*) filename);
 		all_windows_notify_files_changed (list);
 		g_list_free (list);
-	} else 
+	} else
 		data->files_changed_list = g_list_prepend (data->files_changed_list, g_strdup (filename));
 }
 
 
+#define GTK_RESPONSE_TRIM 1
+static gint jpeg_mcu_dialog (GtkWindow *parent)
+{
+ 	GtkWidget *d =  _gtk_message_dialog_new (parent,
+		GTK_DIALOG_MODAL,
+		GTK_STOCK_DIALOG_WARNING,
+		_("This transformation may introduce small image distortions along "
+		"one or more edges, because the image dimensions are not multiples of 8.\n\nThe distortion "
+		"is reversible, however. If the resulting image is unacceptable, simply apply the reverse "
+		"transformation to return to the original image.\n\ngThumb can also discard (or trim) any "
+		"untransformable edge pixels. For practical use, this mode gives the best looking results, "
+		"but the transformation is not strictly lossless anymore.\n\nTo avoid this problem in the "
+		"future, consider disabling the \"Apply physical transform\" option in the rotation dialog."),
+		NULL,
+		_("Trim"), GTK_RESPONSE_TRIM,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		GTK_STOCK_OK, GTK_RESPONSE_OK,
+		NULL);
+
+	gint result = gtk_dialog_run (GTK_DIALOG (d));
+
+ 	gtk_widget_destroy (d);
+
+	if (result == GTK_RESPONSE_DELETE_EVENT)
+		return GTK_RESPONSE_CANCEL;
+	else
+ 		return result;
+}
+
 
 static void
 apply_transformation (DialogData *data,
@@ -305,7 +267,8 @@ apply_transformation (DialogData *data,
 	char             *dir;
 	GnomeVFSFileInfo  info;
 	GtkWindow  	 *window = GTK_WINDOW (data->dialog);
-        
+	GthTransform	  required_transform;
+
 	/* Check directory permissions. */
 
 	dir = remove_level_from_path (path);
@@ -317,18 +280,39 @@ apply_transformation (DialogData *data,
 		g_free (utf8_path);
 		g_free (dir);
 		return;
-	} 
+	}
 	g_free (dir);
 
-	/**/
-
-	if (data->from_exif_data)
-		update_rotation_from_exif_data (path, data->rot_data);
 	gnome_vfs_get_file_info (path, &info, GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-	if (image_is_jpeg (path)) 
-		apply_transformation_jpeg (window, path, data->rot_data);
-	else 
-		apply_transformation_generic (window, path, data->rot_data);
+
+	gboolean jpeg = image_is_jpeg (path);
+	ExifShort orientation = get_exif_tag_short(path, EXIF_TAG_ORIENTATION);
+	required_transform = get_next_transformation (orientation, data->transform);
+
+	if ((jpeg && orientation) &&
+	    !eel_gconf_get_boolean (PREF_ROTATE_RESET_EXIF_ORIENTATION, TRUE)) {
+		// Adjust Exif orientation tag.
+		write_orientation_field (path, required_transform);
+	} else if (jpeg) {
+ 		// Lossless jpeg transform.
+		gint width, height;
+		gdk_pixbuf_get_file_info (get_file_path_from_uri (path), &width, &height);
+		if (!jtransform_perfect_transform (width, height, 8, 8, required_transform)) {
+ 			// Image dimensions are not multiples of the jpeg minimal coding unit (mcu).
+ 			// Warn about possible image distortions along one or more edges.
+ 			gint result = jpeg_mcu_dialog (window);
+			if (result != GTK_RESPONSE_CANCEL) {
+				gboolean trim = (result == GTK_RESPONSE_TRIM);
+				apply_transformation_jpeg (window, path, required_transform, trim);
+			}
+ 		} else {
+			apply_transformation_jpeg (window, path, required_transform, FALSE);
+		}
+	} else {
+		// Generic image transform.
+		apply_transformation_generic (window, path, required_transform);
+	}
+
 	gnome_vfs_set_file_info (path, &info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS|GNOME_VFS_SET_FILE_INFO_OWNER);
 
 	notify_file_changed (data, path, notify_soon);
@@ -345,7 +329,7 @@ apply_transformation_to_all (DialogData *data)
 	GList     *scan;
 	int        i, n;
 
-	gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE, 
+	gui = glade_xml_new (GTHUMB_GLADEDIR "/" PROGRESS_GLADE_FILE,
 			     NULL,
 			     NULL);
 
@@ -359,11 +343,11 @@ apply_transformation_to_all (DialogData *data)
 		gtk_window_set_transient_for (GTK_WINDOW (dialog),
 					      GTK_WINDOW (data->window));
 	else {
-		gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE); 
+		gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 		gtk_window_set_transient_for (GTK_WINDOW (dialog),
 					      GTK_WINDOW (data->dialog));
 	}
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE); 
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_widget_show (dialog);
 
 	while (gtk_events_pending())
@@ -380,14 +364,14 @@ apply_transformation_to_all (DialogData *data)
 
 		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (bar),
 					       (gdouble) (i + 0.5) / n);
-		
+
 		while (gtk_events_pending())
 			gtk_main_iteration();
-		
+
 		apply_transformation (data, scan, FALSE);
 		i++;
 	}
-	
+
 	gtk_widget_destroy (dialog);
 	g_object_unref (gui);
 
@@ -399,13 +383,12 @@ apply_transformation_to_all (DialogData *data)
 
 
 static void
-ok_clicked (GtkWidget  *button, 
+ok_clicked (GtkWidget  *button,
 	    DialogData *data)
 {
 	gboolean to_all;
 
 	to_all = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_apply_to_all_checkbutton));
-	data->from_exif_data = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton));
 
 	if (to_all) {
 		gtk_widget_hide (data->dialog);
@@ -417,25 +400,11 @@ ok_clicked (GtkWidget  *button,
 }
 
 
-static int 
-get_next_rot (int rot)
-{
-	int next_rot [4] = {GTH_TRANSFORM_ROTATE_90, 
-			    GTH_TRANSFORM_ROTATE_180, 
-			    GTH_TRANSFORM_ROTATE_270,
-			    GTH_TRANSFORM_ROTATE_0};
-	return next_rot [rot];
-}
-
-
 static void
-revert_clicked (GtkWidget  *button, 
+revert_clicked (GtkWidget  *button,
 		DialogData *data)
 {
-	data->rot_data->rot_type = GTH_TRANSFORM_ROTATE_0;
-	data->rot_data->tran_type = GTH_TRANSFORM_NONE;
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton), FALSE);
+	data->transform = GTH_TRANSFORM_NONE;
 
 	if (data->original_preview != NULL)
 		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
@@ -443,160 +412,74 @@ revert_clicked (GtkWidget  *button,
 
 
 static void
-rot90_clicked (GtkWidget  *button, 
+transform_clicked_impl (GtkWidget  *button,
+	       DialogData *data,
+	       GthTransform transform)
+{
+	GdkPixbuf *src_pixbuf = NULL;
+	GdkPixbuf *dest_pixbuf = NULL;
+
+	data->transform = get_next_transformation (data->transform, transform);
+
+	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
+
+	if (src_pixbuf == NULL)
+		return;
+
+	dest_pixbuf = _gdk_pixbuf_transform (src_pixbuf, transform);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
+
+	if (dest_pixbuf != NULL)
+		g_object_unref (dest_pixbuf);
+}
+
+
+static void
+rot90_clicked (GtkWidget  *button,
 	       DialogData *data)
 {
-	GdkPixbuf *src_pixbuf;
-	GdkPixbuf *dest_pixbuf;
-
-	if (data->rot_data->tran_type == GTH_TRANSFORM_NONE)
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-	else {
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-	}
-
-	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
-
-	if (src_pixbuf == NULL) 
-		return;
-
-	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
-
-	if (dest_pixbuf != NULL) 
-		g_object_unref (dest_pixbuf);
+	transform_clicked_impl (button, data, GTH_TRANSFORM_ROTATE_90);
 }
 
 
 static void
-rot270_clicked (GtkWidget  *button, 
+rot270_clicked (GtkWidget  *button,
 		DialogData *data)
 {
-	GdkPixbuf *src_pixbuf;
-	GdkPixbuf *dest_pixbuf;
-
-	if (data->rot_data->tran_type == GTH_TRANSFORM_NONE) {
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-	} else
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-
-	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
-
-	if (src_pixbuf == NULL) 
-		return;
-
-	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
-
-	if (dest_pixbuf != NULL) 
-		g_object_unref (dest_pixbuf);
+	transform_clicked_impl (button, data, GTH_TRANSFORM_ROTATE_270);
 }
 
 
 static void
-mirror_clicked (GtkWidget  *button, 
+mirror_clicked (GtkWidget  *button,
 		DialogData *data)
 {
-	GdkPixbuf *src_pixbuf;
-	GdkPixbuf *dest_pixbuf;
-
-	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
-
-	if (src_pixbuf == NULL) 
-		return;
-
-	if (data->rot_data->tran_type == GTH_TRANSFORM_FLIP) {
-		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-
-	} else if (data->rot_data->tran_type == GTH_TRANSFORM_MIRROR)
-		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
-
-	else
-		data->rot_data->tran_type = GTH_TRANSFORM_MIRROR;
-	
-	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
-	
-	if (dest_pixbuf != NULL) 
-		g_object_unref (dest_pixbuf);
+	transform_clicked_impl (button, data, GTH_TRANSFORM_FLIP_H);
 }
 
 
 static void
-flip_clicked (GtkWidget  *button, 
+flip_clicked (GtkWidget  *button,
 	      DialogData *data)
 {
-	GdkPixbuf *src_pixbuf;
-	GdkPixbuf *dest_pixbuf;
-
-	if (data->rot_data->tran_type == GTH_TRANSFORM_MIRROR) {
-		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-		data->rot_data->rot_type = get_next_rot (data->rot_data->rot_type);
-
-	} else if (data->rot_data->tran_type == GTH_TRANSFORM_FLIP)
-		data->rot_data->tran_type = GTH_TRANSFORM_NONE;
-
-	else
-		data->rot_data->tran_type = GTH_TRANSFORM_FLIP;
-
-	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
-
-	if (src_pixbuf == NULL) 
-		return;
-
-	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
-
-	if (dest_pixbuf != NULL) 
-		g_object_unref (dest_pixbuf);
+	transform_clicked_impl (button, data, GTH_TRANSFORM_FLIP_V);
 }
 
 
+static void
+reset_exif_tag_on_rotate_clicked (GtkWidget  *button,
+		              DialogData *data)
+{
+	eel_gconf_set_boolean (PREF_ROTATE_RESET_EXIF_ORIENTATION,
+		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_reset_exif_tag_on_rotate_checkbutton)));
+}
+
 /* called when the "help" button is clicked. */
 static void
-help_cb (GtkWidget  *widget, 
+help_cb (GtkWidget  *widget,
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display ("gthumb", "gthumb-rotate-jpeg", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
-}
-
-
-static void
-from_exif_toggled_cb (GtkToggleButton *button,
-		      DialogData      *data)
-{
-	update_from_exif_data (data);
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-rotate-jpeg");
 }
 
 
@@ -635,13 +518,13 @@ dlg_jpegtran (GthWindow *window)
 
 	if (! data->gui) {
 		g_warning ("Could not find " ROTATE_GLADE_FILE "\n");
-		if (data->file_list != NULL) 
+		if (data->file_list != NULL)
 			path_list_free (data->file_list);
 		g_free (data);
 		return;
 	}
 
-	data->rot_data = rotation_data_new ();
+	data->transform = GTH_TRANSFORM_NONE;
 
 	/* Get the widgets. */
 
@@ -659,7 +542,7 @@ dlg_jpegtran (GthWindow *window)
 	j_v_flip_button = glade_xml_get_widget (data->gui, "j_v_flip_button");
 	j_h_flip_button = glade_xml_get_widget (data->gui, "j_h_flip_button");
 
-	data->j_from_exif_checkbutton = glade_xml_get_widget (data->gui, "j_from_exif_checkbutton");
+	data->j_reset_exif_tag_on_rotate_checkbutton = glade_xml_get_widget (data->gui, "j_reset_exif_tag_on_rotate_checkbutton");
 
 	j_help_button = glade_xml_get_widget (data->gui, "j_help_button");
 	j_cancel_button = glade_xml_get_widget (data->gui, "j_cancel_button");
@@ -678,54 +561,53 @@ dlg_jpegtran (GthWindow *window)
 	gtk_widget_set_sensitive (data->j_apply_to_all_checkbutton,
 				  data->file_list->next != NULL);
 
-#ifndef HAVE_LIBEXIF
-	gtk_widget_set_sensitive (data->j_from_exif_checkbutton, FALSE);
-#endif /* ! HAVE_LIBEXIF */
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->j_reset_exif_tag_on_rotate_checkbutton), eel_gconf_get_boolean (PREF_ROTATE_RESET_EXIF_ORIENTATION, TRUE));
 
 	/* Set the signals handlers. */
-	
+
 	g_signal_connect (G_OBJECT (data->dialog),
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
 
-	g_signal_connect_swapped (G_OBJECT (j_cancel_button), 
+	g_signal_connect_swapped (G_OBJECT (j_cancel_button),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
-	g_signal_connect (G_OBJECT (j_help_button), 
+	g_signal_connect (G_OBJECT (j_help_button),
 			  "clicked",
 			  G_CALLBACK (help_cb),
 			  data);
-	g_signal_connect (G_OBJECT (j_ok_button), 
+	g_signal_connect (G_OBJECT (j_ok_button),
 			  "clicked",
 			  G_CALLBACK (ok_clicked),
 			  data);
-	g_signal_connect (G_OBJECT (j_revert_button), 
+	g_signal_connect (G_OBJECT (j_revert_button),
 			  "clicked",
 			  G_CALLBACK (revert_clicked),
 			  data);
-	g_signal_connect (G_OBJECT (j_rot_90_button), 
+	g_signal_connect (G_OBJECT (j_rot_90_button),
 			  "clicked",
 			  G_CALLBACK (rot90_clicked),
 			  data);
-	g_signal_connect (G_OBJECT (j_rot_270_button), 
+	g_signal_connect (G_OBJECT (j_rot_270_button),
 			  "clicked",
 			  G_CALLBACK (rot270_clicked),
 			  data);
-	g_signal_connect (G_OBJECT (j_v_flip_button), 
+	g_signal_connect (G_OBJECT (j_v_flip_button),
 			  "clicked",
 			  G_CALLBACK (mirror_clicked),
 			  data);
-	g_signal_connect (G_OBJECT (j_h_flip_button), 
+	g_signal_connect (G_OBJECT (j_h_flip_button),
 			  "clicked",
 			  G_CALLBACK (flip_clicked),
 			  data);
-	g_signal_connect (G_OBJECT (data->j_from_exif_checkbutton),
-			  "toggled",
-			  G_CALLBACK (from_exif_toggled_cb),
+        g_signal_connect (G_OBJECT (data->j_reset_exif_tag_on_rotate_checkbutton),
+ 			  "clicked",
+                          G_CALLBACK (reset_exif_tag_on_rotate_clicked),
 			  data);
 
+
 	data->loader = (ImageLoader*)image_loader_new (NULL, FALSE);
 
 	g_signal_connect (G_OBJECT (data->loader),
@@ -742,7 +624,7 @@ dlg_jpegtran (GthWindow *window)
 	all_windows_remove_monitor ();
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window));
-	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE); 
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 	gtk_widget_show_all (data->dialog);
 
 	load_current_image (data);
@@ -751,8 +633,7 @@ dlg_jpegtran (GthWindow *window)
 
 void
 dlg_apply_jpegtran (GthWindow    *window,
-		    GthTransform  rot_type,
-		    GthTransform  tran_type)
+		    GthTransform  transform)
 {
 	DialogData  *data;
 	GList       *list;
@@ -769,36 +650,7 @@ dlg_apply_jpegtran (GthWindow    *window,
 	data->window = window;
 	data->file_list = list;
 	data->current_image = list;
-
-	data->rot_data = rotation_data_new();
-	data->rot_data->rot_type = rot_type;
-	data->rot_data->tran_type = tran_type;
-
-	apply_transformation_to_all (data);
-}
-
-
-
-void
-dlg_apply_jpegtran_from_exif (GthWindow *window)
-{
-	DialogData  *data;
-	GList       *list;
-
-	list = gth_window_get_file_list_selection (window);
-	if (list == NULL) {
-		g_warning ("No file selected.");
-		return;
-	}
-
-	all_windows_remove_monitor ();
-
-	data = g_new0 (DialogData, 1);
-	data->window = window;
-	data->file_list = list;
-	data->current_image = list;
-	data->from_exif_data = TRUE;
-	data->rot_data = rotation_data_new();
+	data->transform = transform;
 
 	apply_transformation_to_all (data);
 }
