@@ -30,6 +30,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "gth-browser.h"
+#include "gth-utils.h"
 #include "gtk-utils.h"
 #include "gconf-utils.h"
 #include "glib-utils.h"
@@ -37,6 +38,7 @@
 #include "file-utils.h"
 #include "dlg-file-utils.h"
 #include "preferences.h"
+#include "gth-sort-utils.h"
 
 enum {
 	RS_OLDNAME_COLUMN,
@@ -44,8 +46,14 @@ enum {
 	RS_NUM_COLUMNS
 };
 
-int            sort_method_to_idx[] = { -1, 0, -1, 1, 2, 3 };
-GthSortMethod  idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME, GTH_SORT_METHOD_BY_SIZE, GTH_SORT_METHOD_BY_TIME, GTH_SORT_METHOD_MANUAL };
+static int           sort_method_to_idx[] = { -1, 0, 1, 2, 3, 4, 5 };
+static GthSortMethod idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME,
+                                              GTH_SORT_METHOD_BY_PATH,
+                                              GTH_SORT_METHOD_BY_SIZE,
+                                              GTH_SORT_METHOD_BY_TIME,
+                                              GTH_SORT_METHOD_BY_EXIF_DATE,
+                                              GTH_SORT_METHOD_BY_COMMENT,
+                                              GTH_SORT_METHOD_MANUAL};
 
 
 #define GLADE_FILE "gthumb_tools.glade"
@@ -65,17 +73,19 @@ typedef struct {
 	GList         *original_file_list;
 	GList         *file_list;
 	GList         *new_names_list;
+	GthSortMethod  sort_method;
+	gboolean       single_file;
 } DialogData;
 
 
 /* called when the main dialog is closed. */
 static void
-destroy_cb (GtkWidget  *widget, 
+destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	if (data->file_list != NULL)
 		g_list_free (data->file_list);
-	file_data_list_free (data->original_file_list); 
+	file_data_list_free (data->original_file_list);
 	if (data->new_names_list != NULL) {
 		g_list_foreach (data->new_names_list, (GFunc) g_free, NULL);
 		g_list_free (data->new_names_list);
@@ -87,7 +97,7 @@ destroy_cb (GtkWidget  *widget,
 
 /* called when the "ok" button is pressed. */
 static void
-ok_clicked_cb (GtkWidget  *widget, 
+ok_clicked_cb (GtkWidget  *widget,
 	       DialogData *data)
 {
 	GList *old_names = NULL;
@@ -98,22 +108,25 @@ ok_clicked_cb (GtkWidget  *widget,
 	/* Save options */
 
 	template = gtk_entry_get_text (GTK_ENTRY (data->rs_template_entry));
-	eel_gconf_set_string (PREF_RENAME_SERIES_TEMPLATE, template);
+	if (strchr (template, '/') != NULL) {
+		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
+				       _("The name \"%s\" is not valid because it contains the character \"/\". " "Please use a different name."), template);
+		return;
+	}
 
-	eel_gconf_set_integer (PREF_RENAME_SERIES_START_AT, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->rs_start_at_spinbutton)));
-
-	pref_set_rename_sort_order (idx_to_sort_method [gtk_combo_box_get_active (GTK_COMBO_BOX (data->rs_sort_combobox))]);
-
-	eel_gconf_set_boolean (PREF_RENAME_SERIES_REVERSE, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->rs_reverse_checkbutton)));
-
-	/**/
+	if (! data->single_file) {
+		eel_gconf_set_string (PREF_RENAME_SERIES_TEMPLATE, template);
+		eel_gconf_set_integer (PREF_RENAME_SERIES_START_AT, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->rs_start_at_spinbutton)));
+		pref_set_rename_sort_order (idx_to_sort_method [gtk_combo_box_get_active (GTK_COMBO_BOX (data->rs_sort_combobox))]);
+		eel_gconf_set_boolean (PREF_RENAME_SERIES_REVERSE, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->rs_reverse_checkbutton)));
+	}
 
 	for (o_scan = data->file_list, n_scan = data->new_names_list; o_scan && n_scan; o_scan = o_scan->next, n_scan = n_scan->next) {
-		FileData *fdata     = o_scan->data;
-		char *old_full_path = fdata->path;
-		char *new_name      = n_scan->data;
-		char *old_path      = remove_level_from_path (old_full_path);
-		char *new_full_path;
+		FileData *fdata         = o_scan->data;
+		char     *old_full_path = fdata->path;
+		char     *new_name      = n_scan->data;
+		char     *old_path      = remove_level_from_path (old_full_path);
+		char     *new_full_path;
 
 		new_full_path = g_strconcat (old_path, "/", new_name, NULL);
 		g_free (old_path);
@@ -129,64 +142,123 @@ ok_clicked_cb (GtkWidget  *widget,
 }
 
 
-static gint
-sort_by_name (gconstpointer  ptr1,
-              gconstpointer  ptr2)
+static int
+comp_func_name (gconstpointer  ptr1,
+		gconstpointer  ptr2)
 {
-	const FileData *fdata1 = ptr1, *fdata2 = ptr2;
-	return strcasecmp (fdata1->name, fdata2->name); 
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
+
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_filename_but_ignore_path (fd1->name, fd2->name);
 }
 
 
-static gint
-sort_by_size (gconstpointer  ptr1,
-              gconstpointer  ptr2)
+static int
+comp_func_size (gconstpointer  ptr1,
+		gconstpointer  ptr2)
 {
-	const FileData *fdata1 = ptr1, *fdata2 = ptr2;
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
 
-	if (fdata1->size == fdata2->size)
-		return sort_by_name (ptr1, ptr2);
-	else if (fdata1->size > fdata2->size)
-		return 1;
-	else
-		return -1;
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_size_then_name (fd1->size, fd2->size, fd1->path, fd2->path);
 }
 
 
-static gint
-sort_by_time (gconstpointer  ptr1,
-              gconstpointer  ptr2)
+static int
+comp_func_time (gconstpointer  ptr1,
+		gconstpointer  ptr2)
 {
-	const FileData *fdata1 = ptr1, *fdata2 = ptr2;
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
 
-	if (fdata1->mtime == fdata2->mtime)
-		return sort_by_name (ptr1, ptr2);
-	else if (fdata1->mtime > fdata2->mtime)
-		return 1;
-	else
-		return -1;
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_filetime_then_name (fd1->mtime, fd2->mtime,
+						 fd1->path, fd2->path);
 }
 
 
-static gint
-sort_by_manual_order (gconstpointer  ptr1,
-		      gconstpointer  ptr2)
+static int
+comp_func_exif_date (gconstpointer  ptr1,
+		     gconstpointer  ptr2)
+{
+	FileData *fd1 = (FileData *) ptr1, *fd2 = (FileData *) ptr2;
+
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_exiftime_then_name (fd1, fd2);
+}
+
+
+static int
+comp_func_path (gconstpointer  ptr1,
+		gconstpointer  ptr2)
+{
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
+
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_full_path (fd1->path, fd2->path);
+}
+
+
+static int
+comp_func_comment (gconstpointer ptr1, gconstpointer ptr2)
+{
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
+
+	return gth_sort_by_comment_then_name (fd1->comment, fd2->comment,
+					      fd1->path, fd2->path);
+}
+
+
+static int
+comp_func_none (gconstpointer  ptr1,
+		gconstpointer  ptr2)
 {
 	return -1;
 }
 
 
 static GCompareFunc
-get_compare_func_from_idx (int column_index)
+get_sortfunc (DialogData *data)
 {
-	static GCompareFunc compare_funcs[4] = {
-		sort_by_name,
-		sort_by_size,
-		sort_by_time,
-		sort_by_manual_order
-	};
+        GCompareFunc func;
 
-	return compare_funcs [column_index % 4];
+        switch (data->sort_method) {
+        case GTH_SORT_METHOD_BY_NAME:
+                func = comp_func_name;
+                break;
+        case GTH_SORT_METHOD_BY_TIME:
+                func = comp_func_time;
+                break;
+        case GTH_SORT_METHOD_BY_SIZE:
+                func = comp_func_size;
+                break;
+        case GTH_SORT_METHOD_BY_PATH:
+                func = comp_func_path;
+                break;
+        case GTH_SORT_METHOD_BY_COMMENT:
+                func = comp_func_comment;
+                break;
+        case GTH_SORT_METHOD_BY_EXIF_DATE:
+                func = comp_func_exif_date;
+                break;
+        case GTH_SORT_METHOD_NONE:
+                func = comp_func_none;
+                break;
+        default:
+                func = comp_func_none;
+                break;
+        }
+
+        return func;
 }
 
 
@@ -217,18 +289,17 @@ static void
 update_list (DialogData *data)
 {
 	GList  *scan, *on_scan, *nn_scan;
-	int     idx;
 	int     start_at;
 	char  **template;
 	const char   *template_s;
 
-	idx = gtk_combo_box_get_active (GTK_COMBO_BOX (data->rs_sort_combobox));
+	data->sort_method = idx_to_sort_method [gtk_combo_box_get_active (GTK_COMBO_BOX (data->rs_sort_combobox))];
 
 	if (data->file_list != NULL)
 		g_list_free (data->file_list);
 	data->file_list = g_list_copy (data->original_file_list);
-	data->file_list = g_list_sort (data->file_list, get_compare_func_from_idx (idx));
-	
+	data->file_list = g_list_sort (data->file_list, get_sortfunc (data));
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->rs_reverse_checkbutton)))
 		data->file_list = g_list_reverse (data->file_list);
 
@@ -253,11 +324,10 @@ update_list (DialogData *data)
 		char     *name2;
 		char     *name3;
 		char     *name4;
-		char     *extension;
+		char     *extension = NULL;
 		char     *new_name;
 
 		name1 = _g_get_name_from_template (template, start_at++);
-
 		utf8_txt = g_filename_to_utf8 (name_wo_ext, -1, 0, 0, 0);
 		name2 = _g_substitute_pattern (name1, 'f', utf8_txt);
 		g_free (name_wo_ext);
@@ -273,8 +343,10 @@ update_list (DialogData *data)
 		name4 = _g_substitute_pattern (name3, 's', image_size);
 		g_free (image_size);
 
-		extension = g_filename_to_utf8 (strrchr (fdata->name, '.'), -1, 0, 0, 0);
-		new_name = g_strconcat (name4, extension, NULL);
+		if (strrchr (fdata->name, '.') != NULL)
+			extension = g_filename_to_utf8 (strrchr (fdata->name, '.'), -1, 0, 0, 0);
+
+		new_name = _g_substitute_pattern (name4, 'e', extension);
 
 		data->new_names_list = g_list_prepend (data->new_names_list, g_filename_from_utf8 (new_name, -1, 0, 0, 0));
 
@@ -298,9 +370,9 @@ update_list (DialogData *data)
 		GtkTreeIter  iter;
 		char        *utf8_on;
 		char        *utf8_nn;
-		
+
 		gtk_list_store_append (data->rs_list_model, &iter);
-		
+
 		utf8_on = g_filename_display_name (fdata->name);
 		utf8_nn = g_filename_display_name (new_name);
 		gtk_list_store_set (data->rs_list_model, &iter,
@@ -325,38 +397,13 @@ update_list_cb (GtkWidget  *widget,
 
 /* called when the "help" button is clicked. */
 static void
-help_cb (GtkWidget  *widget, 
+help_cb (GtkWidget  *widget,
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display ("gthumb", "gthumb-rename-series", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-rename-series");
 }
 
 
-
 void
 dlg_rename_series (GthBrowser *browser)
 {
@@ -367,7 +414,6 @@ dlg_rename_series (GthBrowser *browser)
 	GList             *list;
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
-	char              *svalue;
 	gboolean           reorderable;
 	int                idx;
 
@@ -380,6 +426,7 @@ dlg_rename_series (GthBrowser *browser)
 	data = g_new0 (DialogData, 1);
 
 	data->original_file_list = list;
+	data->single_file = (data->original_file_list->next == NULL);
 	data->file_list = NULL;
 	data->new_names_list = NULL;
 	data->browser = browser;
@@ -408,7 +455,7 @@ dlg_rename_series (GthBrowser *browser)
 	/* Set widgets data. */
 
 	data->rs_list_model = gtk_list_store_new (RS_NUM_COLUMNS,
-						  G_TYPE_STRING, 
+						  G_TYPE_STRING,
 						  G_TYPE_STRING);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (data->rs_list_treeview),
 				 GTK_TREE_MODEL (data->rs_list_model));
@@ -430,59 +477,59 @@ dlg_rename_series (GthBrowser *browser)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (data->rs_list_treeview),
 				     column);
 
-	gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
-				   _("by size"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
-				   _("by modified time"));
-	if (reorderable)
-		gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
-					   _("manual order"));
-
-	/**/
-
-	svalue = eel_gconf_get_string (PREF_RENAME_SERIES_TEMPLATE, "###");
-	gtk_entry_set_text (GTK_ENTRY (data->rs_template_entry), svalue);
-	g_free (svalue);
-
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->rs_start_at_spinbutton), eel_gconf_get_integer (PREF_RENAME_SERIES_START_AT, 1));
-
+        gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
+                                   _("by path"));
+        gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
+                                   _("by size"));
+        gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
+                                   _("by file modified time"));
+        gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
+                                   _("by Exif DateTime tag"));
+        gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
+                                   _("by comment"));
+        if (reorderable)
+                gtk_combo_box_append_text (GTK_COMBO_BOX (data->rs_sort_combobox),
+                                           _("manual order"));
 	idx = sort_method_to_idx [pref_get_rename_sort_order ()];
 	if (!reorderable && (sort_method_to_idx[GTH_SORT_METHOD_MANUAL] == idx))
 		idx = sort_method_to_idx[GTH_SORT_METHOD_BY_NAME];
 	gtk_combo_box_set_active (GTK_COMBO_BOX (data->rs_sort_combobox), idx);
-
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->rs_reverse_checkbutton), eel_gconf_get_boolean (PREF_RENAME_SERIES_REVERSE, FALSE));
 
-	update_list (data);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->rs_start_at_spinbutton), eel_gconf_get_integer (PREF_RENAME_SERIES_START_AT, 1));
+
+	gtk_widget_set_sensitive (data->rs_sort_combobox, ! data->single_file);
+	gtk_widget_set_sensitive (data->rs_reverse_checkbutton, ! data->single_file);
+	gtk_widget_set_sensitive (data->rs_start_at_spinbutton, ! data->single_file);
 
 	/* Set the signals handlers. */
-	
-	g_signal_connect (G_OBJECT (data->dialog), 
+
+	g_signal_connect (G_OBJECT (data->dialog),
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
-	g_signal_connect (G_OBJECT (btn_ok), 
+	g_signal_connect (G_OBJECT (btn_ok),
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-	g_signal_connect_swapped (G_OBJECT (btn_cancel), 
+	g_signal_connect_swapped (G_OBJECT (btn_cancel),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  G_OBJECT (data->dialog));
-	g_signal_connect (G_OBJECT (help_button), 
+	g_signal_connect (G_OBJECT (help_button),
 			  "clicked",
 			  G_CALLBACK (help_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (data->rs_template_entry), 
+	g_signal_connect (G_OBJECT (data->rs_template_entry),
 			  "changed",
 			  G_CALLBACK (update_list_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->rs_start_at_spinbutton), 
+	g_signal_connect (G_OBJECT (data->rs_start_at_spinbutton),
 			  "value_changed",
 			  G_CALLBACK (update_list_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->rs_sort_combobox), 
+	g_signal_connect (G_OBJECT (data->rs_sort_combobox),
 			  "changed",
 			  G_CALLBACK (update_list_cb),
 			  data);
@@ -493,8 +540,28 @@ dlg_rename_series (GthBrowser *browser)
 
 	/* run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog),
 				      GTK_WINDOW (browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 	gtk_widget_show (data->dialog);
+
+	if (data->single_file) {
+		FileData   *fd = data->original_file_list->data;
+		const char *last_dot;
+
+		gtk_entry_set_text (GTK_ENTRY (data->rs_template_entry), fd->utf8_name);
+		last_dot = strrchr (fd->name, '.');
+		if (last_dot == NULL)
+			last_dot = fd->name + strlen (fd->name) - 1;
+		gtk_editable_select_region (GTK_EDITABLE (data->rs_template_entry), 0, last_dot - fd->name);
+	}
+	else {
+		char *svalue;
+
+		svalue = eel_gconf_get_string (PREF_RENAME_SERIES_TEMPLATE, "###");
+		gtk_entry_set_text (GTK_ENTRY (data->rs_template_entry), svalue);
+		g_free (svalue);
+	}
+
+	update_list (data);
 }
