@@ -29,6 +29,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-url.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -43,22 +44,25 @@
 #include "pixbuf-utils.h"
 #include "gconf-utils.h"
 #include "gth-browser.h"
+#include "gth-utils.h"
 #include "glib-utils.h"
 
-static int           sort_method_to_idx[] = { -1, 0, 1, 2, 3, 4 };
+static int           sort_method_to_idx[] = { -1, 0, 1, 2, 3, 4, 5 };
 static GthSortMethod idx_to_sort_method[] = { GTH_SORT_METHOD_BY_NAME, 
 					      GTH_SORT_METHOD_BY_PATH, 
 					      GTH_SORT_METHOD_BY_SIZE, 
 					      GTH_SORT_METHOD_BY_TIME,
+					      GTH_SORT_METHOD_BY_EXIF_DATE,
+					      GTH_SORT_METHOD_BY_COMMENT,
 					      GTH_SORT_METHOD_MANUAL};
-static int           idx_to_resize_width[] = { 320, 640, 800, 1024, 1280 };
-static int           idx_to_resize_height[] = { 200, 480, 600, 768, 960 };
+static int           idx_to_resize_width[] = { 320, 320, 640, 640, 800, 800, 1024, 1024, 1280, 1280 };
+static int           idx_to_resize_height[] = { 200, 320, 480, 640, 600, 800, 768, 1024, 960, 1280 };
 
 
 #define str_void(x) (((x) == NULL) || (*(x) == 0))
 #define GLADE_EXPORTER_FILE "gthumb_web_exporter.glade"
 #define MAX_PREVIEW_SIZE 220
-
+#define DEFAULT_ALBUM_THEME "Wiki"
 
 typedef struct {
 	GthBrowser         *browser;
@@ -83,6 +87,9 @@ typedef struct {
 
 	GtkWidget          *wa_rows_spinbutton;
 	GtkWidget          *wa_cols_spinbutton;
+	GtkWidget          *wa_single_index_checkbutton;
+	GtkWidget          *wa_rows_hbox;
+	GtkWidget          *wa_cols_hbox;
 	GtkWidget          *wa_sort_images_combobox;
 	GtkWidget          *wa_reverse_order_checkbutton;
 
@@ -110,6 +117,15 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
+/* called when the "help" button is clicked. */
+static void
+help_cb (GtkWidget  *widget,
+         DialogData *data)
+{
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-web-album");
+}
+
+
 static void
 export (GtkWidget  *widget,
 	DialogData *data)
@@ -146,6 +162,8 @@ export (GtkWidget  *widget,
 
 	eel_gconf_set_integer (PREF_WEB_ALBUM_COLUMNS, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->wa_cols_spinbutton)));
 
+	eel_gconf_set_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->wa_single_index_checkbutton)));
+
 	pref_set_web_album_sort_order (idx_to_sort_method [gtk_combo_box_get_active (GTK_COMBO_BOX (data->wa_sort_images_combobox))]);
 
 	eel_gconf_set_boolean (PREF_WEB_ALBUM_REVERSE, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->wa_reverse_order_checkbutton)));
@@ -159,6 +177,11 @@ export (GtkWidget  *widget,
 	theme = _gtk_entry_get_filename_text (GTK_ENTRY (data->wa_theme_combo_entry));
 	eel_gconf_set_string (PREF_WEB_ALBUM_THEME, theme);
 
+	if (strcmp (theme, "") == 0) {
+		g_free (location);
+		return;
+	}
+
 	/**/
 
 	if (! dlg_check_folder (GTH_WINDOW (data->browser), location)) {
@@ -181,6 +204,7 @@ export (GtkWidget  *widget,
 						idx_to_resize_height[gtk_option_menu_get_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu))]);
 
 	catalog_web_exporter_set_row_col (exporter, eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS, 4), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS, 4));
+	catalog_web_exporter_set_single_index (exporter, eel_gconf_get_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, FALSE));
 	
 	catalog_web_exporter_set_sorted (exporter, pref_get_web_album_sort_order (), eel_gconf_get_boolean (PREF_WEB_ALBUM_REVERSE, FALSE));
 	catalog_web_exporter_set_header (exporter, header);
@@ -241,20 +265,22 @@ static void show_album_theme_cb (GtkWidget *widget, DialogData *data);
 
 
 static int
-get_idx_from_resize_width (int width)
+get_idx_from_size (int width, int height)
 {
+	int idx;
 	if (width == 320)
-		return 0;
+		idx = 0;
 	else if (width == 640)
-		return 1;
+		idx = 1;
 	else if (width == 800)
-		return 2;
+		idx = 2;
 	else if (width == 1024)
-		return 3;
+		idx = 3;
 	else if (width == 1280)
-		return 4;
+		idx = 4;
 	else
-		return 1;
+		idx = 1;
+	return 2 * idx + (width == height? 1 : 0);
 }
 
 
@@ -274,6 +300,90 @@ resize_image_toggled_cb (GtkToggleButton *button,
 }
 
 
+static void
+single_index_toggled_cb (GtkToggleButton *button,
+			 DialogData      *data)
+{
+	gtk_widget_set_sensitive (data->wa_rows_hbox, !gtk_toggle_button_get_active (button));
+}
+
+ 
+
+static gboolean
+theme_present (const char *theme_name,
+	       const char *theme_dir)
+{
+	GnomeVFSResult  result;
+	GList          *file_list = NULL;
+	GList          *scan;
+	gboolean        found = FALSE;
+
+	if (theme_name == NULL)
+		return FALSE;
+
+	if (theme_dir != NULL)
+		result = gnome_vfs_directory_list_load (&file_list, 
+							theme_dir, 
+							GNOME_VFS_FILE_INFO_DEFAULT);
+	else
+		result = GNOME_VFS_ERROR_NOT_A_DIRECTORY;
+	
+	if (result == GNOME_VFS_OK) 
+		for (scan = file_list; scan; scan = scan->next) {
+			GnomeVFSFileInfo *info = scan->data;
+
+			if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
+				continue;
+
+			if ((strcmp (info->name, ".") == 0)
+			    || (strcmp (info->name, "..") == 0))
+				continue;
+
+			if (strcmp (info->name, theme_name) == 0) {
+				found = TRUE;
+				break;
+			}
+		}
+
+	return found;
+}
+
+
+static char *
+get_default_theme (void)
+{
+	char     *current_theme;
+	char     *theme_dir;
+	gboolean  found = FALSE;
+
+	current_theme = eel_gconf_get_string (PREF_WEB_ALBUM_THEME, DEFAULT_ALBUM_THEME);
+
+	theme_dir = g_build_path (G_DIR_SEPARATOR_S,
+				  g_get_home_dir (),
+				  ".gnome2",
+				  "gthumb/albumthemes",
+				  NULL);
+	found = theme_present (current_theme, theme_dir);
+	if (!found) {
+		g_free (theme_dir);
+		theme_dir = g_build_path (G_DIR_SEPARATOR_S,
+					  GTHUMB_DATADIR,
+					  "gthumb/albumthemes",
+					  NULL);
+		found = theme_present (current_theme, theme_dir);
+	}
+
+	g_free (theme_dir);
+
+	if (! found) {
+		g_free (current_theme);
+		return g_strdup ("");
+	}
+
+	return current_theme;
+}
+
+
 /* create the main dialog. */
 void
 dlg_web_exporter (GthBrowser *browser)
@@ -326,6 +436,9 @@ dlg_web_exporter (GthBrowser *browser)
 
 	data->wa_rows_spinbutton = glade_xml_get_widget (data->gui, "wa_rows_spinbutton");
 	data->wa_cols_spinbutton = glade_xml_get_widget (data->gui, "wa_cols_spinbutton");
+	data->wa_single_index_checkbutton = glade_xml_get_widget (data->gui, "wa_single_index_checkbutton");
+	data->wa_rows_hbox = glade_xml_get_widget (data->gui, "wa_rows_hbox");
+	data->wa_cols_hbox = glade_xml_get_widget (data->gui, "wa_cols_hbox");
 	data->wa_sort_images_combobox = glade_xml_get_widget (data->gui, "wa_sort_images_combobox");
 	data->wa_reverse_order_checkbutton = glade_xml_get_widget (data->gui, "wa_reverse_order_checkbutton");
 
@@ -359,12 +472,16 @@ dlg_web_exporter (GthBrowser *browser)
 
 	gtk_widget_set_sensitive (data->wa_resize_images_options_hbox, eel_gconf_get_boolean (PREF_WEB_ALBUM_RESIZE_IMAGES, FALSE));
 
-	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu), get_idx_from_resize_width (eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_WIDTH, 640)));
+	gtk_option_menu_set_history (GTK_OPTION_MENU (data->wa_resize_images_optionmenu), get_idx_from_size (eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_WIDTH, 640), eel_gconf_get_integer (PREF_WEB_ALBUM_RESIZE_HEIGHT, 480)));
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_rows_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_ROWS, 4));
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->wa_cols_spinbutton), eel_gconf_get_integer (PREF_WEB_ALBUM_COLUMNS, 4));
 
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->wa_single_index_checkbutton), eel_gconf_get_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, FALSE));
+
+	gtk_widget_set_sensitive (data->wa_rows_hbox, !eel_gconf_get_boolean (PREF_WEB_ALBUM_SINGLE_INDEX, FALSE));
+
 	/**/
 
 	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
@@ -372,7 +489,11 @@ dlg_web_exporter (GthBrowser *browser)
 	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
 				   _("by size"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
-				   _("by modified time"));
+				   _("by file modified time"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
+				   _("by Exif DateTime tag"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
+                                   _("by comment"));
 	if (reorderable)
 		gtk_combo_box_append_text (GTK_COMBO_BOX (data->wa_sort_images_combobox),
 					   _("manual order"));
@@ -394,7 +515,7 @@ dlg_web_exporter (GthBrowser *browser)
 	gtk_entry_set_text (GTK_ENTRY (data->wa_footer_entry), svalue);
 	g_free (svalue);
 
-	svalue = eel_gconf_get_string (PREF_WEB_ALBUM_THEME, "Clean");
+	svalue =get_default_theme();
 	_gtk_entry_set_filename_text (GTK_ENTRY (data->wa_theme_combo_entry), svalue);
 	g_free (svalue);
 
@@ -418,6 +539,10 @@ dlg_web_exporter (GthBrowser *browser)
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
+        g_signal_connect (G_OBJECT (btn_help),
+                          "clicked",
+                          G_CALLBACK (help_cb),
+                          data);
 	g_signal_connect_swapped (G_OBJECT (btn_cancel), 
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
@@ -426,7 +551,6 @@ dlg_web_exporter (GthBrowser *browser)
 			  "clicked",
 			  G_CALLBACK (export),
 			  data);
-
 	g_signal_connect (G_OBJECT (data->wa_select_theme_button), 
 			  "clicked",
 			  G_CALLBACK (show_album_theme_cb),
@@ -441,6 +565,11 @@ dlg_web_exporter (GthBrowser *browser)
 			  G_CALLBACK (resize_image_toggled_cb),
 			  data);
 
+	g_signal_connect (G_OBJECT (data->wa_single_index_checkbutton),
+			  "toggled",
+			  G_CALLBACK (single_index_toggled_cb),
+			  data);
+
 	g_signal_connect (G_OBJECT (data->exporter), 
 			  "web_exporter_done",
 			  G_CALLBACK (export_done),
@@ -475,7 +604,7 @@ dlg_web_exporter (GthBrowser *browser)
 }
 
 
-
+
 
 typedef struct {
 	DialogData         *data;
@@ -954,6 +1083,8 @@ typedef struct {
 	GtkWidget          *dialog;
 
 	GtkWidget          *c_comment_checkbutton;
+	GtkWidget          *c_place_checkbutton;
+	GtkWidget          *c_date_time_checkbutton;
 	GtkWidget          *c_imagedim_checkbutton;
 	GtkWidget          *c_filename_checkbutton;
 	GtkWidget          *c_filesize_checkbutton;
@@ -990,6 +1121,10 @@ caption_dialog__ok_clicked (GtkWidget         *widget,
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_comment_checkbutton)))
 		caption |= GTH_CAPTION_COMMENT;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_place_checkbutton)))
+		caption |= GTH_CAPTION_PLACE;
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_date_time_checkbutton)))
+		caption |= GTH_CAPTION_DATE_TIME;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_imagedim_checkbutton)))
 		caption |= GTH_CAPTION_IMAGE_DIM;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cdata->c_filename_checkbutton)))
@@ -1035,12 +1170,8 @@ show_caption_dialog_cb (GtkWidget       *widget,
 	CaptionDialogData *cdata;
 	GtkWidget         *ok_button;
 	GtkWidget         *cancel_button;
-#ifdef HAVE_LIBEXIF
 	const char        *gconf_key;
 	GthCaptionFields   caption = 0;
-#else
-	GtkWidget         *vbox;
-#endif
 	cdata = g_new (CaptionDialogData, 1);
 
 	cdata->data = tdata->data;
@@ -1058,6 +1189,8 @@ show_caption_dialog_cb (GtkWidget       *widget,
 
 	cdata->dialog = glade_xml_get_widget (cdata->gui, "caption_dialog");
 	cdata->c_comment_checkbutton = glade_xml_get_widget (cdata->gui, "c_comment_checkbutton");
+	cdata->c_place_checkbutton = glade_xml_get_widget (cdata->gui, "c_place_checkbutton");
+	cdata->c_date_time_checkbutton = glade_xml_get_widget (cdata->gui, "c_date_time_checkbutton");
 	cdata->c_imagedim_checkbutton = glade_xml_get_widget (cdata->gui, "c_imagedim_checkbutton");
 	cdata->c_filename_checkbutton = glade_xml_get_widget (cdata->gui, "c_filename_checkbutton");
 	cdata->c_filesize_checkbutton = glade_xml_get_widget (cdata->gui, "c_filesize_checkbutton");
@@ -1090,8 +1223,6 @@ show_caption_dialog_cb (GtkWidget       *widget,
 
 	/* Set widgets data. */
 
-#ifdef HAVE_LIBEXIF
-
 	if (cdata->thumbnail_caption) 
 		gconf_key = PREF_WEB_ALBUM_INDEX_CAPTION;
 	 else 
@@ -1100,6 +1231,10 @@ show_caption_dialog_cb (GtkWidget       *widget,
 
 	if (caption & GTH_CAPTION_COMMENT)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_comment_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_PLACE)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_place_checkbutton), TRUE);
+	if (caption & GTH_CAPTION_DATE_TIME)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_date_time_checkbutton), TRUE);
 	if (caption & GTH_CAPTION_IMAGE_DIM)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_imagedim_checkbutton), TRUE);
 	if (caption & GTH_CAPTION_FILE_NAME)
@@ -1123,10 +1258,6 @@ show_caption_dialog_cb (GtkWidget       *widget,
 	if (caption & GTH_CAPTION_EXIF_CAMERA_MODEL)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cdata->c_exif_camera_model_checkbutton), TRUE);
 
-#else /* ! HAVE_LIBEXIF */
-	vbox = glade_xml_get_widget (cdata->gui, "c_exif_data_vbox");
-	gtk_widget_hide (vbox);
-#endif /* ! HAVE_LIBEXIF */
 
 	/* Run dialog. */
 
