@@ -31,7 +31,7 @@
 #include "gth-browser-actions-entries.h"
 
 
-static const gchar *main_ui_info = 
+static const gchar *main_ui_info =
 "<ui>"
 "  <menubar name='MenuBar'>"
 "    <menu name='File' action='FileMenu'>"
@@ -88,6 +88,9 @@ static const gchar *main_ui_info =
 "      <menuitem action='View_Fullscreen'/>"
 "      <menuitem action='Tools_Slideshow'/>"
 "      <separator/>"
+"      <menuitem action='View_AsSlides'/>"
+"      <menuitem action='View_AsList'/>"
+"      <separator/>"
 "      <menuitem action='View_ShowFolders'/>"
 "      <menuitem action='View_ShowCatalogs'/>"
 "      <menuitem action='View_ShowImage'/>"
@@ -97,6 +100,8 @@ static const gchar *main_ui_info =
 "        <menuitem action='SortByPath'/>"
 "        <menuitem action='SortBySize'/>"
 "        <menuitem action='SortByTime'/>"
+"	 <menuitem action='SortByExifDate'/>"
+"        <menuitem action='SortByComment'/>"
 "        <menuitem action='SortManual'/>"
 "        <separator/>"
 "        <menuitem action='SortReversed'/>"
@@ -107,6 +112,7 @@ static const gchar *main_ui_info =
 "        <menuitem action='View_ZoomOut'/>"
 "        <menuitem action='View_Zoom100'/>"
 "        <menuitem action='View_ZoomFit'/>"
+"        <menuitem action='View_ZoomWidth'/>"
 "        <separator/>"
 "        <menuitem action='View_ZoomQualityHigh'/>"
 "        <menuitem action='View_ZoomQualityLow'/>"
@@ -159,6 +165,8 @@ static const gchar *main_ui_info =
 "        <separator/>"
 "        <menuitem action='AlterImage_Posterize'/>"
 "      </menu>"
+"      <separator/>"
+"      <menuitem action='AlterImage_RedeyeRemoval'/>"
 "    </menu>"
 "    <menu name='Tools' action='ToolsMenu'>"
 "      <menu name='Wallpaper' action='ToolsWallpaperMenu'>"
@@ -171,6 +179,8 @@ static const gchar *main_ui_info =
 "      </menu>"
 "      <separator/>"
 "      <menuitem action='Tools_JPEGRotate'/>"
+"      <menuitem action='Tools_ResetExif'/>"
+"      <separator/>"
 "      <menuitem action='Tools_ScaleSeries'/>"
 "      <menuitem action='Tools_ConvertFormat'/>"
 "      <menuitem action='Tools_ChangeDate'/>"
@@ -186,15 +196,12 @@ static const gchar *main_ui_info =
 "    </menu>"
 "  </menubar>"
 "  <toolbar name='ToolBar'>"
-"    <toolitem action='View_ShowFolders'/>"
-"    <toolitem action='View_ShowCatalogs'/>"
-"    <toolitem action='View_ShowImage'/>"
-"    <separator/>"
 "    <placeholder name='ModeCommands'/>"
 "  </toolbar>"
 "  <popup name='FilePopup'>"
 "    <menuitem action='File_ViewImage'/>"
 "    <menuitem action='File_OpenWith'/>"
+"    <menuitem action='File_Print'/>"
 "    <menuitem action='Go_ToContainer'/>"
 "    <separator/>"
 "    <menu action='ToolsWallpaperMenu'>"
@@ -218,10 +225,15 @@ static const gchar *main_ui_info =
 "    <separator/>"
 "    <menuitem action='Edit_AddToCatalog'/>"
 "    <menuitem action='Edit_RemoveFromCatalog'/>"
+"    <separator/>"
+"    <menuitem action='File_ImageProp'/>"
 "  </popup>"
 "  <popup name='ImagePopup'>"
 "    <menuitem action='File_ViewImage'/>"
 "    <menuitem action='Image_OpenWith'/>"
+"    <menuitem action='File_Save'/>"
+"    <menuitem action='File_SaveAs'/>"
+"    <menuitem action='File_Print'/>"
 "    <menuitem action='Go_ToContainer'/>"
 "    <separator/>"
 "    <menuitem action='View_Fullscreen'/>"
@@ -292,13 +304,13 @@ static const gchar *main_ui_info =
 "    <menuitem action='Tools_JPEGRotate_Right'/>"
 "    <menuitem action='Tools_JPEGRotate_Left'/>"
 "    <separator/>"
-"    <menuitem action='Tools_JPEGRotate_Auto'/>"
+"    <menuitem action='Tools_ResetExif'/>"
 "  </popup>"
 "  <accelerator name='ControlEqualAccel' action='ControlEqual'/>"
 "</ui>";
 
 
-static const gchar *folder_ui_info = 
+static const gchar *folder_ui_info =
 "<ui>"
 "  <menubar name='MenuBar'>"
 "    <menu name='File' action='FileMenu'>"
@@ -306,12 +318,12 @@ static const gchar *folder_ui_info =
 "        <menuitem action='EditCurrentDir_New'/>"
 "        <menu name='Folder' action='FileFolderMenu'>"
 "          <menuitem action='EditCurrentDir_Open'/>"
-"          <separator name='sep01'/>"
+"          <separator/>"
 "          <menuitem action='EditCurrentDir_Rename'/>"
 "          <menuitem action='EditCurrentDir_Delete'/>"
 "          <menuitem action='EditCurrentDir_Copy'/>"
 "          <menuitem action='EditCurrentDir_Move'/>"
-"          <separator name='sep02'/>"
+"          <separator/>"
 "          <menuitem action='EditCurrentDir_Categories'/>"
 "        </menu>"
 "      </placeholder>"
@@ -320,7 +332,7 @@ static const gchar *folder_ui_info =
 "</ui>";
 
 
-static const gchar *catalog_ui_info = 
+static const gchar *catalog_ui_info =
 "<ui>"
 "  <menubar name='MenuBar'>"
 "    <menu name='File' action='FileMenu'>"
@@ -363,37 +375,39 @@ static const gchar *browser_ui_info =
 "<ui>"
 "  <toolbar name='ToolBar'>"
 "    <placeholder name='ModeCommands'>"
+"      <toolitem action='View_ShowFolders'/>"
+"      <toolitem action='View_ShowCatalogs'/>"
+"      <separator/>"
 "      <toolitem action='View_Fullscreen'/>"
 "      <toolitem action='Tools_Slideshow'/>"
 "      <separator/>"
-"      <toolitem action='Tools_FindImages'/>"
+"      <toolitem action='ToolBar_FindImages'/>"
 "      <separator/>"
-"      <toolitem action='Edit_EditComment'/>"
-"      <toolitem action='Edit_EditCategories'/>"
+"      <toolitem action='ToolBar_EditComment'/>"
+"      <toolitem action='ToolBar_EditCategories'/>"
 "    </placeholder>"
 "  </toolbar>"
 "</ui>";
 
-
 static const gchar *viewer_ui_info =
 "<ui>"
 "  <toolbar name='ToolBar'>"
 "    <placeholder name='ModeCommands'>"
+"      <toolitem action='View_CloseImageMode'/>"
+"      <toolitem action='File_Save'/>"
+"      <separator/>"
 "      <toolitem action='View_PrevImage'/>"
 "      <toolitem action='View_NextImage'/>"
-"      <separator name='sep01'/>"
-"      <toolitem action='Edit_EditComment'/>"
-"      <toolitem action='Edit_EditCategories'/>"
-"      <separator name='sep02'/>"
+"      <separator/>"
 "      <toolitem action='View_Fullscreen'/>"
-"      <separator name='sep03'/>"
+"      <separator/>"
 "      <toolitem action='View_ZoomIn'/>"
 "      <toolitem action='View_ZoomOut'/>"
 "      <toolitem action='View_Zoom100'/>"
 "      <toolitem action='View_ZoomFit'/>"
+"      <toolitem action='View_ZoomWidth'/>"
 "    </placeholder>"
 "  </toolbar>"
 "</ui>";
 
-
 #endif /* GTH_BROWSER_UI_H */
