@@ -26,7 +26,7 @@
 #include "gth-file-list.h"
 #include "gth-file-view.h"
 #include "catalog-list.h"
-#include "dir-list.h"
+#include "gth-dir-list.h"
 #include "gth-window.h"
 #include "typedefs.h"
 
@@ -72,6 +72,8 @@ void            gth_browser_go_to_directory              (GthBrowser        *bro
 const char *    gth_browser_get_current_directory        (GthBrowser        *browser);
 void            gth_browser_go_to_catalog_directory      (GthBrowser        *browser,
 							  const gchar       *dir_path);
+void            gth_browser_show_catalog_directory       (GthBrowser        *browser,
+			                                  const char        *catalog_dir);							  
 void            gth_browser_go_to_catalog                (GthBrowser        *browser,
 							  const gchar       *catalog_path);
 const char *    gth_browser_get_current_catalog          (GthBrowser        *browser);
@@ -99,9 +101,12 @@ void            gth_browser_set_sort_type                (GthBrowser        *bro
 
 GthFileList *   gth_browser_get_file_list                (GthBrowser        *browser);
 GthFileView *   gth_browser_get_file_view                (GthBrowser        *browser);
-DirList *       gth_browser_get_dir_list                 (GthBrowser        *browser);
+GthDirList *    gth_browser_get_dir_list                 (GthBrowser        *browser);
 CatalogList *   gth_browser_get_catalog_list             (GthBrowser        *browser);
 
+void            gth_browser_load_uri                     (GthBrowser        *browser,
+							  const char        *uri);
+
 /* non-modal dialogs */
 
 void            gth_browser_set_bookmarks_dlg            (GthBrowser        *browser,
